/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
// Forked Cosmo 3.3.7
// Variables
// --------------------------------------------------

// == Colors
//
// ## Gray and brand colors for use across Bootstrap.

@gray-base: @darkest; // superset-var
@gray-darker: lighten(@gray-base, 13.5%);
@gray-dark: lighten(@gray-base, 20%);
@bs-gray: lighten(@gray-base, 33.5%);
@bs-gray-light: lighten(@gray-base, 70%);
@gray-lighter: lighten(@gray-base, 95%);

@brand-primary: @primary-color; // superset-var
@brand-success: @success; // superset-var
@brand-info: @info; // superset-var
@brand-warning: @warning; // superset-var
@brand-danger: @danger; // superset-var

// == Scaffolding
//
// ## Settings for some of the most global styles.

// ** Background color for `<body>`.
@body-bg: @gray-bg; // superset-var
// ** Global text color on `<body>`.
@text-color: @gray-dark;

// ** Global textual link color.
@link-color: @link;
// ** Link hover color set via `darken()` function.
@link-hover-color: @link-hover;
// ** Link hover decoration.
@link-hover-decoration: underline;

// == Typography
//
// ## Font, line-height, and color for body text, headings, and more.

@font-size-large: ceil((@font-size-base * 1.25)); // ~18px
@font-size-small: ceil((@font-size-base * 0.85)); // ~12px

@font-size-h1: floor((@font-size-base * 2.6)); // ~36px
@font-size-h2: floor((@font-size-base * 2.15)); // ~30px
@font-size-h3: ceil((@font-size-base * 1.7)); // ~24px
@font-size-h4: ceil((@font-size-base * 1.25)); // ~18px
@font-size-h5: @font-size-base;
@font-size-h6: ceil((@font-size-base * 0.85)); // ~12px

// ** Computed "line-height" (`font-size` * `line-height`) for use with `margin`, `padding`, etc.
@line-height-computed: floor((@font-size-base * @line-height-base));

// ** By default, this inherits from the `<body>`.
@headings-font-family: @font-family-base;
@headings-font-weight: @font-weight-normal; // superset-var
@headings-line-height: @line-height-tight;
@headings-color: inherit;

// == Iconography
//
// ## Specify custom location and filename of the included Glyphicons icon font. Useful for those including Bootstrap via Bower.

// ** Load fonts from this directory.
@icon-font-path: '../fonts/';
// ** File name for all font files.
@icon-font-name: 'glyphicons-halflings-regular';
// ** Element ID within SVG icon file.
@icon-font-svg-id: 'glyphicons_halflingsregular';

// == Components
//
// ## Define common padding and border radius sizes and more. Values based on 14px text and 1.428 line-height (~20px to start).

@padding-base-vertical: 10px;
@padding-base-horizontal: 18px;

@padding-large-vertical: 18px;
@padding-large-horizontal: 30px;

@padding-small-vertical: 5px;
@padding-small-horizontal: 10px;

@padding-xs-vertical: 1px;
@padding-xs-horizontal: 5px;

@line-height-large: 1.3333333; // extra decimals for Win 8.1 Chrome
@line-height-small: 1.5;

@border-radius-base: @border-radius-normal;
@border-radius-large: 4px;
@border-radius-small: 4px;

// ** Global color for active items (e.g., navs or dropdowns).
@component-active-color: @lightest; // superset-var
// ** Global background color for active items (e.g., navs or dropdowns).
@component-active-bg: @brand-primary;

// ** Width of the `border` for generating carets that indicate dropdowns.
@caret-width-base: 4px;
// ** Carets increase slightly in size for larger components.
@caret-width-large: 5px;

// == Tables
//
// ## Customizes the `.table` component with basic values, each used across all table variations.

// ** Padding for `<th>`s and `<td>`s.
@table-cell-padding: 8px;
// ** Padding for cells in `.table-condensed`.
@table-condensed-cell-padding: 5px;

// ** Default background color used for all tables.
@table-bg: transparent;
// ** Background color used for `.table-striped`.
@table-bg-accent: fade(@gray-bg, @opacity-medium-light); // superset-var
// ** Background color used for `.table-hover`.
@table-bg-hover: @gray-bg; // superset-var
@table-bg-active: @table-bg-hover;

// ** Border color for table and cell borders.
@table-border-color: @gray-light; // superset-var

// == Buttons
//
// ## For each of Bootstrap's buttons, define text, background and border color.

@btn-font-weight: normal;

@btn-primary-color: @lightest;
@btn-primary-bg: @brand-primary;
@btn-primary-border: @brand-primary;

@btn-default-color: @bs-gray;
@btn-default-bg: @lightest;
@btn-default-border: @bs-gray-light;

@btn-success-color: @btn-primary-color;
@btn-success-bg: @brand-success;
@btn-success-border: @btn-success-bg;

@btn-info-color: @btn-primary-color;
@btn-info-bg: @brand-info;
@btn-info-border: @btn-info-bg;

@btn-warning-color: @btn-primary-color;
@btn-warning-bg: @brand-warning;
@btn-warning-border: @btn-warning-bg;

@btn-danger-color: @btn-primary-color;
@btn-danger-bg: @brand-danger;
@btn-danger-border: @btn-danger-bg;

@btn-link-disabled-color: @bs-gray-light;

// Allows for customizing button radius independently from global border radius
@btn-border-radius-base: @border-radius-base;
@btn-border-radius-large: @border-radius-large;
@btn-border-radius-small: @border-radius-small;

// == Forms
//
// ##

// ** `<input>` background color
@input-bg: @lightest; // superset-var
// ** `<input disabled>` background color
@input-bg-disabled: @gray-lighter;

// ** Text color for `<input>`s
@input-color: @text-color;
// ** `<input>` border color
@input-border: @gray-light; // superset-var

// TODO: Rename `@input-border-radius` to `@input-border-radius-base` in v4
// ** Default `.form-control` border radius
// This has no effect on `<select>`s in some browsers, due to the limited stylability of `<select>`s in CSS.
@input-border-radius: @border-radius-base;
// ** Large `.form-control` border radius
@input-border-radius-large: @border-radius-large;
// ** Small `.form-control` border radius
@input-border-radius-small: @border-radius-small;

// ** Border color for inputs on focus
@input-border-focus: @indicator-color; // superset-var

// ** Placeholder text color
@input-color-placeholder: @bs-gray-light;

// ** Default `.form-control` height
@input-height-base: (@line-height-computed + (@padding-base-vertical * 2) + 2);
// ** Large `.form-control` height
@input-height-large: (
  ceil(@font-size-large * @line-height-large) + (@padding-large-vertical * 2) +
    2
);
// ** Small `.form-control` height
@input-height-small: (
  floor(@font-size-small * @line-height-small) + (@padding-small-vertical * 2) +
    2
);

// ** `.form-group` margin
@form-group-margin-bottom: 10px;

@legend-color: @text-color;
@legend-border-color: @gray-bg;

// ** Background color for textual input addons
@input-group-addon-bg: @gray-lighter;
// ** Border color for textual input addons
@input-group-addon-border-color: @input-border;

// ** Disabled cursor for form controls and buttons.
@cursor-disabled: not-allowed;

// == Dropdowns
//
// ## Dropdown menu container and contents.

// ** Background for the dropdown menu.
@dropdown-bg: @lightest; // superset-var
// ** Dropdown menu `border-color`.
@dropdown-border: fade(@darkest, @opacity-light); // superset-var
// ** Dropdown menu `border-color` **for IE8**.
@dropdown-fallback-border: @gray-light; // superset-var
// ** Divider color for between dropdown items.
@dropdown-divider-bg: @gray-bg; // superset-var

// ** Dropdown link text color.
@dropdown-link-color: @gray-dark;
// ** Hover color for dropdown links.
@dropdown-link-hover-color: @lightest; // superset-var
// ** Hover background for dropdown links.
@dropdown-link-hover-bg: @component-active-bg;

// ** Active dropdown menu item text color.
@dropdown-link-active-color: @lightest; // superset-var
// ** Active dropdown menu item background color.
@dropdown-link-active-bg: @component-active-bg;

// ** Disabled dropdown menu item background color.
@dropdown-link-disabled-color: @bs-gray-light;

// ** Text color for headers within dropdown menus.
@dropdown-header-color: @bs-gray-light;

// ** Deprecated `@dropdown-caret-color` as of v3.1.0
@dropdown-caret-color: @darkest; // superset-var

//-- Z-index master list
//
// Warning: Avoid customizing these values. They're used for a bird's eye view
// of components dependent on the z-axis and are designed to all work together.
//
// Note: These variables are not generated into the Customizer.

@zindex-navbar: 1000;
@zindex-dropdown: 1000;
@zindex-popover: 1060;
@zindex-tooltip: 1070;
@zindex-navbar-fixed: 1030;
@zindex-modal-background: 1040;
@zindex-modal: 1050;

// == Media queries breakpoints
//
// ## Define the breakpoints at which your layout will change, adapting to different screen sizes.

// Extra small screen / phone
// ** Deprecated `@screen-xs` as of v3.0.1
@screen-xs: 480px;
// ** Deprecated `@screen-xs-min` as of v3.2.0
@screen-xs-min: @screen-xs;
// ** Deprecated `@screen-phone` as of v3.0.1
@screen-phone: @screen-xs-min;

// Small screen / tablet
// ** Deprecated `@screen-sm` as of v3.0.1
@screen-sm: 768px;
@screen-sm-min: @screen-sm;
// ** Deprecated `@screen-tablet` as of v3.0.1
@screen-tablet: @screen-sm-min;

// Medium screen / desktop
// ** Deprecated `@screen-md` as of v3.0.1
@screen-md: 992px;
@screen-md-min: @screen-md;
// ** Deprecated `@screen-desktop` as of v3.0.1
@screen-desktop: @screen-md-min;

// Large screen / wide desktop
// ** Deprecated `@screen-lg` as of v3.0.1
@screen-lg: 1200px;
@screen-lg-min: @screen-lg;
// ** Deprecated `@screen-lg-desktop` as of v3.0.1
@screen-lg-desktop: @screen-lg-min;

// So media queries don't overlap when required, provide a maximum
@screen-xs-max: (@screen-sm-min - 1);
@screen-sm-max: (@screen-md-min - 1);
@screen-md-max: (@screen-lg-min - 1);

// == Grid system
//
// ## Define your custom responsive grid.

// ** Number of columns in the grid.
@grid-columns: 12;
// ** Padding between columns. Gets divided in half for the left and right.
@grid-gutter-width: 20px;
// Navbar collapse
// ** Point at which the navbar becomes uncollapsed.
@grid-float-breakpoint: @screen-sm-min;
// ** Point at which the navbar begins collapsing.
@grid-float-breakpoint-max: (@grid-float-breakpoint - 1);

// == Container sizes
//
// ## Define the maximum width of `.container` for different screen sizes.

// Small screen / tablet
@container-tablet: (720px + @grid-gutter-width);
// ** For `@screen-sm-min` and up.
@container-sm: @container-tablet;

// Medium screen / desktop
@container-desktop: (940px + @grid-gutter-width);
// ** For `@screen-md-min` and up.
@container-md: @container-desktop;

// Large screen / wide desktop
@container-large-desktop: (1140px + @grid-gutter-width);
// ** For `@screen-lg-min` and up.
@container-lg: @container-large-desktop;

// == Navbar
//
// ##

// Basics of a navbar
@navbar-height: 50px;
@navbar-margin-bottom: @line-height-computed;
@navbar-border-radius: @border-radius-base;
@navbar-padding-horizontal: floor((@grid-gutter-width / 2));
@navbar-padding-vertical: ((@navbar-height - @line-height-computed) / 2);
@navbar-collapse-max-height: 340px;

@navbar-default-color: @lightest; // superset-var
@navbar-default-bg: @gray-darker;
@navbar-default-border: darken(@navbar-default-bg, 6.5%);

// Navbar links
@navbar-default-link-color: @lightest; // superset-var
@navbar-default-link-hover-color: @lightest; // superset-var
@navbar-default-link-hover-bg: darken(@navbar-default-bg, 10%);
@navbar-default-link-active-color: @navbar-default-link-hover-color;
@navbar-default-link-active-bg: @navbar-default-link-hover-bg;
@navbar-default-link-disabled-color: @gray-light; // superset-var
@navbar-default-link-disabled-bg: transparent;

// Navbar brand label
@navbar-default-brand-color: @navbar-default-link-color;
@navbar-default-brand-hover-color: @lightest; // superset-var
@navbar-default-brand-hover-bg: none;

// Navbar toggle
@navbar-default-toggle-hover-bg: @navbar-default-link-hover-bg;
@navbar-default-toggle-icon-bar-bg: @lightest; // superset-var
@navbar-default-toggle-border-color: transparent;

// === Inverted navbar
// Reset inverted navbar basics
@navbar-inverse-color: @gray-dark;
@navbar-inverse-bg: @lightest; // superset-var
@navbar-inverse-border: transparent;

// Inverted navbar links
@navbar-inverse-link-color: @gray-dark;
@navbar-inverse-link-hover-color: @gray-dark;
@navbar-inverse-link-hover-bg: darken(@navbar-inverse-bg, 10%);
@navbar-inverse-link-active-color: @navbar-inverse-link-hover-color;
@navbar-inverse-link-active-bg: @navbar-inverse-link-hover-bg;
@navbar-inverse-link-disabled-color: @gray-lighter;
@navbar-inverse-link-disabled-bg: transparent;

// Inverted navbar brand label
@navbar-inverse-brand-color: @navbar-inverse-link-color;
@navbar-inverse-brand-hover-color: @gray-darker;
@navbar-inverse-brand-hover-bg: none;

// Inverted navbar toggle
@navbar-inverse-toggle-hover-bg: @navbar-inverse-link-hover-bg;
@navbar-inverse-toggle-icon-bar-bg: @lightest; // superset-var
@navbar-inverse-toggle-border-color: transparent;

// == Navs
//
// ##

// === Shared nav styles
@nav-link-padding: 10px 15px;
@nav-link-hover-bg: @gray-lighter;

@nav-disabled-link-color: @bs-gray-light;
@nav-disabled-link-hover-color: @bs-gray-light;

// == Tabs
@nav-tabs-border-color: @gray-light; // superset-var

@nav-tabs-link-hover-border-color: @gray-lighter;

@nav-tabs-active-link-hover-bg: @body-bg;
@nav-tabs-active-link-hover-color: @bs-gray;
@nav-tabs-active-link-hover-border-color: @gray-light; // superset-var

@nav-tabs-justified-link-border-color: @gray-light; // superset-var
@nav-tabs-justified-active-link-border-color: @body-bg;

// == Pills
@nav-pills-border-radius: @border-radius-base;
@nav-pills-active-link-hover-bg: @component-active-bg;
@nav-pills-active-link-hover-color: @component-active-color;

// == Pagination
//
// ##

@pagination-color: @link-color;
@pagination-bg: @lightest; // superset-var
@pagination-border: @gray-light; // superset-var

@pagination-hover-color: @link-hover-color;
@pagination-hover-bg: @gray-lighter;
@pagination-hover-border: @gray-light; // superset-var

@pagination-active-color: @bs-gray-light;
@pagination-active-bg: @gray-bg; // superset-var
@pagination-active-border: @gray-light; // superset-var

@pagination-disabled-color: @bs-gray-light;
@pagination-disabled-bg: @lightest; // superset-var
@pagination-disabled-border: @gray-light; // superset-var

// == Pager
//
// ##

@pager-bg: @pagination-bg;
@pager-border: @pagination-border;
@pager-border-radius: @border-radius-base;

@pager-hover-bg: @pagination-hover-bg;

@pager-active-bg: @pagination-active-bg;
@pager-active-color: @pagination-active-color;

@pager-disabled-color: @bs-gray-light;

// == Jumbotron
//
// ##

@jumbotron-padding: 30px;
@jumbotron-color: inherit;
@jumbotron-bg: @gray-lighter;
@jumbotron-heading-color: inherit;
@jumbotron-font-size: ceil((@font-size-base * 1.5));
@jumbotron-heading-font-size: ceil((@font-size-base * 4.5));

// == Form states and alerts
//
// ## Define colors for form feedback states and, by default, alerts.

@state-success-text: darken(@brand-success, 20%);
@state-success-bg: lighten(@brand-success, 35%);
@state-success-border: darken(spin(@state-success-bg, -10), 5%);

@state-info-text: darken(@brand-info, 22%);
@state-info-bg: lighten(@brand-info, 20%);
@state-info-border: darken(spin(@state-info-bg, -10), 7%);

@state-warning-text: darken(@brand-warning, 30%);
@state-warning-bg: lighten(@brand-warning, 20%);
@state-warning-border: darken(spin(@state-warning-bg, -10), 3%);

@state-danger-text: darken(@brand-danger, 25%);
@state-danger-bg: lighten(@brand-danger, 22%);
@state-danger-border: darken(spin(@state-danger-bg, -10), 3%);

// == Tooltips
//
// ##

// ** Tooltip max width
@tooltip-max-width: 200px;
// ** Tooltip text color
@tooltip-color: @lightest; // superset-var
// ** Tooltip background color
@tooltip-bg: @darkest; // superset-var
@tooltip-opacity: 0.9;

// ** Tooltip arrow width
@tooltip-arrow-width: 5px;
// ** Tooltip arrow color
@tooltip-arrow-color: @tooltip-bg;

// == Popovers
//
// ##

// ** Popover body background color
@popover-bg: @lightest; // superset-var
// ** Popover maximum width
@popover-max-width: 276px;
// ** Popover border color
@popover-border-color: fade(@darkest, @opacity-light); // superset-var
// ** Popover fallback border color
@popover-fallback-border-color: @gray-light; // superset-var

// ** Popover title background color
@popover-title-bg: darken(@popover-bg, 3%);

// ** Popover arrow width
@popover-arrow-width: 10px;
// ** Popover arrow color
@popover-arrow-color: @popover-bg;

// ** Popover outer arrow width
@popover-arrow-outer-width: (@popover-arrow-width + 1);
// ** Popover outer arrow color
@popover-arrow-outer-color: fadein(@popover-border-color, 5%);
// ** Popover outer arrow fallback color
@popover-arrow-outer-fallback-color: darken(
  @popover-fallback-border-color,
  20%
);

// == Labels
//
// ##

// ** Default label background color
@label-default-bg: @bs-gray-light;
// ** Primary label background color
@label-primary-bg: @brand-primary;
// ** Success label background color
@label-success-bg: @brand-success;
// ** Info label background color
@label-info-bg: @brand-info;
// ** Warning label background color
@label-warning-bg: @brand-warning;
// ** Danger label background color
@label-danger-bg: @brand-danger;

// ** Default label text color
@label-color: @lightest; // superset-var
// ** Default text color of a linked label
@label-link-hover-color: @lightest; // superset-var

// == Modals
//
// ##

// ** Padding applied to the modal body
@modal-inner-padding: 20px;

// ** Padding applied to the modal title
@modal-title-padding: 15px;
// ** Modal title line-height
@modal-title-line-height: @line-height-base;

// ** Background color of modal content area
@modal-content-bg: @lightest; // superset-var
// ** Modal content border color
@modal-content-border-color: transparent;
// ** Modal content border color **for IE8**
@modal-content-fallback-border-color: @gray; // superset-var

// ** Modal backdrop background color
@modal-backdrop-bg: @darkest; // superset-var
// ** Modal backdrop opacity
@modal-backdrop-opacity: 0.5;
// ** Modal header border color
@modal-header-border-color: @gray-bg; // superset-var
// ** Modal footer border color
@modal-footer-border-color: @modal-header-border-color;

@modal-lg: 900px;
@modal-md: 600px;
@modal-sm: 300px;

// == Alerts
//
// ## Define alert colors, border radius, and padding.

@alert-padding: 15px;
@alert-border-radius: @border-radius-base;
@alert-link-font-weight: bold;

@alert-success-bg: @state-success-bg;
@alert-success-text: @state-success-text;
@alert-success-border: @state-success-border;

@alert-info-bg: @state-info-bg;
@alert-info-text: @state-info-text;
@alert-info-border: @state-info-border;

@alert-warning-bg: @state-warning-bg;
@alert-warning-text: @state-warning-text;
@alert-warning-border: @state-warning-border;

@alert-danger-bg: @state-danger-bg;
@alert-danger-text: @state-danger-text;
@alert-danger-border: @state-danger-border;

// == Progress bars
//
// ##

// ** Background color of the whole progress component
@progress-bg: @gray-light; // superset-var
// ** Progress bar text color
@progress-bar-color: @lightest; // superset-var
// ** Variable for setting rounded corners on progress bar.
@progress-border-radius: @border-radius-base;

// ** Default progress bar color
@progress-bar-bg: @brand-primary;
// ** Success progress bar color
@progress-bar-success-bg: @brand-success;
// ** Warning progress bar color
@progress-bar-warning-bg: @brand-warning;
// ** Danger progress bar color
@progress-bar-danger-bg: @brand-danger;
// ** Info progress bar color
@progress-bar-info-bg: @brand-info;

// == List group
//
// ##

// ** Background color on `.list-group-item`
@list-group-bg: @lightest; // superset-var
// ** `.list-group-item` border color
@list-group-border: @gray-light; // superset-var
// ** List group border radius
@list-group-border-radius: @border-radius-base;

// ** Background color of single list items on hover
@list-group-hover-bg: @gray-bg; // superset-var
// ** Text color of active list items
@list-group-active-color: @component-active-color;
// ** Background color of active list items
@list-group-active-bg: @component-active-bg;
// ** Border color of active list elements
@list-group-active-border: @list-group-border;
// ** Text color for content within active list items
@list-group-active-text-color: lighten(@list-group-active-bg, 40%);

// ** Text color of disabled list items
@list-group-disabled-color: @bs-gray-light;
// ** Background color of disabled list items
@list-group-disabled-bg: @gray-lighter;
// ** Text color for content within disabled list items
@list-group-disabled-text-color: @list-group-disabled-color;

@list-group-link-color: #555;
@list-group-link-hover-color: @list-group-link-color;
@list-group-link-heading-color: @almost-black; // superset-var

// == Panels
//
// ##

@panel-bg: @lightest; // superset-var
@panel-body-padding: 15px;
@panel-heading-padding: 10px 15px;
@panel-footer-padding: @panel-heading-padding;
@panel-border-radius: @border-radius-base;

// ** Border color for elements within panels
@panel-inner-border: @gray-light; // superset-var
@panel-footer-bg: @gray-bg; // superset-var

@panel-default-text: @gray-dark;
@panel-default-border: transparent;
@panel-default-heading-bg: @lightest; // superset-var

@panel-primary-text: @lightest; // superset-var
@panel-primary-border: transparent;
@panel-primary-heading-bg: @brand-primary;

@panel-success-text: @state-success-text;
@panel-success-border: transparent;
@panel-success-heading-bg: @state-success-bg;

@panel-info-text: @state-info-text;
@panel-info-border: transparent;
@panel-info-heading-bg: @state-info-bg;

@panel-warning-text: @state-warning-text;
@panel-warning-border: transparent;
@panel-warning-heading-bg: @state-warning-bg;

@panel-danger-text: @state-danger-text;
@panel-danger-border: transparent;
@panel-danger-heading-bg: @state-danger-bg;

// == Thumbnails
//
// ##

// ** Padding around the thumbnail image
@thumbnail-padding: 4px;
// ** Thumbnail background color
@thumbnail-bg: @body-bg;
// ** Thumbnail border color
@thumbnail-border: @gray-light; // superset-var
// ** Thumbnail border radius
@thumbnail-border-radius: @border-radius-base;

// ** Custom text color for thumbnail captions
@thumbnail-caption-color: @text-color;
// ** Padding around the thumbnail caption
@thumbnail-caption-padding: 9px;

// == Wells
//
// ##

@well-bg: @gray-bg; // superset-var
@well-border: darken(@well-bg, 7%);

// == Badges
//
// ##

@badge-color: @lightest; // superset-var
// ** Linked badge text color on hover
@badge-link-hover-color: @lightest; // superset-var
@badge-bg: @brand-primary;

// ** Badge text color in active nav link
@badge-active-color: @link-color;
// ** Badge background color in active nav link
@badge-active-bg: @lightest; // superset-var

@badge-font-weight: bold;
@badge-line-height: 1;
@badge-border-radius: 10px;

// == Breadcrumbs
//
// ##

@breadcrumb-padding-vertical: 8px;
@breadcrumb-padding-horizontal: 15px;
// ** Breadcrumb background color
@breadcrumb-bg: @gray-bg; // superset-var
// ** Breadcrumb text color
@breadcrumb-color: @gray-light; // superset-var
// ** Text color of current page in the breadcrumb
@breadcrumb-active-color: @bs-gray-light;
// ** Textual separator for between breadcrumb elements
@breadcrumb-separator: '/';

// == Carousel
//
// ##

@carousel-text-shadow: 0 1px 2px fade(@darkest, @opacity-light); // superset-var

@carousel-control-color: @lightest; // superset-var
@carousel-control-width: 15%;
@carousel-control-opacity: 0.5;
@carousel-control-font-size: 20px;

@carousel-indicator-active-bg: @lightest; // superset-var
@carousel-indicator-border-color: @lightest; // superset-var

@carousel-caption-color: @lightest; // superset-var

// == Close
//
// ##

@close-font-weight: bold;
@close-color: @lightest; // superset-var
@close-text-shadow: 0 1px 0 @lightest; // superset-var

// == Code
//
// ##

@code-color: darken(@info, @colorstop-one);
@code-bg: @gray-bg; // superset-var

@kbd-color: @lightest; // superset-var
@kbd-bg: @almost-black; // superset-var

@pre-bg: @gray-bg; // superset-var
@pre-color: @gray-dark;
@pre-border-color: @gray-light; // superset-var
@pre-scrollable-max-height: 340px;

// == Type
//
// ##

// ** Horizontal offset for forms and lists.
@component-offset-horizontal: 180px;
// ** Text muted color
@text-muted: @bs-gray-light;
// ** Abbreviations and acronyms border color
@abbr-border-color: @bs-gray-light;
// ** Headings small color
@headings-small-color: @bs-gray-light;
// ** Blockquote small color
@blockquote-small-color: @bs-gray-light;
// ** Blockquote font size
@blockquote-font-size: (@font-size-base * 1.25);
// ** Blockquote border color
@blockquote-border-color: @gray-lighter;
// ** Page header border color
@page-header-border-color: @gray-lighter;
// ** Width of horizontal description list titles
@dl-horizontal-offset: @component-offset-horizontal;
// ** Point at which .dl-horizontal becomes horizontal
@dl-horizontal-breakpoint: @grid-float-breakpoint;
// ** Horizontal line color.
@hr-border: @gray-lighter;
