/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 100;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Thin.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Thin.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 100;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-ThinItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-ThinItalic.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 200;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-ExtraLight.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-ExtraLight.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 200;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-ExtraLightItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-ExtraLightItalic.woff?v=3.12')
      format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 300;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Light.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Light.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 300;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-LightItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-LightItalic.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 400;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Regular.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Regular.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 400;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Italic.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Italic.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 500;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Medium.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Medium.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 500;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-MediumItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-MediumItalic.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 600;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-SemiBold.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-SemiBold.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 600;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-SemiBoldItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-SemiBoldItalic.woff?v=3.12')
      format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 700;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Bold.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Bold.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 700;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-BoldItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-BoldItalic.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 800;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-ExtraBold.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-ExtraBold.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 800;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-ExtraBoldItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-ExtraBoldItalic.woff?v=3.12')
      format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: normal;
  font-weight: 900;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-Black.woff2?v=3.12') format('woff2'),
    url('../../../fonts/InterUI/Inter-Black.woff?v=3.12') format('woff');
}
@font-face {
  font-family: 'Inter';
  font-style: italic;
  font-weight: 900;
  font-display: swap;
  src: url('../../../fonts/InterUI/Inter-BlackItalic.woff2?v=3.12')
      format('woff2'),
    url('../../../fonts/InterUI/Inter-BlackItalic.woff?v=3.12') format('woff');
}

/* -------------------------------------------------------
Variable font.
Usage:

  html { font-family: 'Inter', sans-serif; }
  @supports (font-variation-settings: normal) {
    html { font-family: 'Inter var', sans-serif; }
  }
*/
@font-face {
  font-family: 'Inter var';
  font-weight: 100 900;
  font-display: swap;
  font-style: normal;
  font-named-instance: 'Regular';
  src: url('../../../fonts/InterUI/Inter-roman.var.woff2?v=3.12')
    format('woff2');
}
@font-face {
  font-family: 'Inter var';
  font-weight: 100 900;
  font-display: swap;
  font-style: italic;
  font-named-instance: 'Italic';
  src: url('../../../fonts/InterUI/Inter-italic.var.woff2?v=3.12')
    format('woff2');
}

/* --------------------------------------------------------------------------
[EXPERIMENTAL] Multi-axis, single variable font.

Slant axis is not yet widely supported (as of February 2019) and thus this
multi-axis single variable font is opt-in rather than the default.

When using this, you will probably need to set font-variation-settings
explicitly, e.g.

  * { font-variation-settings: "slnt" 0deg }
  .italic { font-variation-settings: "slnt" 10deg }

*/
@font-face {
  font-family: 'Inter var experimental';
  font-weight: 100 900;
  font-display: swap;
  font-style: oblique 0deg 10deg;
  src: url('../../../fonts/InterUI/Inter.var.woff2?v=3.12') format('woff2');
}
