.BootstrapSliderWrapper .slider-selection{background:#efefef}.BootstrapSliderWrapper .slider-handle{background:#b3b3b3}.play-slider{display:flex;height:40px;width:100%;margin:0}.play-slider-controls{flex:0 0 80px;text-align:middle}.play-slider-scrobbler{flex:1}.slider.slider-horizontal{width:100%!important}.slider-button{color:#b3b3b3;margin-right:5px}div.slider>div.tooltip.tooltip-main.top.in{margin-left:0!important}