.reactable-pagination td{padding:15px 0 0!important}.reactable-pagination a:focus{text-decoration:none;color:#333;outline:1}.reactable-next-page,.reactable-page-button,.reactable-previous-page{background:#fff;border-radius:4px;border:1px solid #cfd8dc;color:#333;display:inline-block;font-size:12px;margin-right:5px;padding:5px 10px;text-align:center;text-decoration:none;vertical-align:middle;white-space:nowrap}.reactable-next-page:hover,.reactable-page-button:hover,.reactable-previous-page:hover{background-color:#f5f5f5;border-color:#879399;color:#333;text-decoration:none}.reactable-current-page{border:1px solid #cfd8dc;color:#333;font-weight:700;pointer-events:none;opacity:.65}.active .color-bar.badge-20,.dashboard-component-chart-holder:hover .color-bar.badge-20,.dashboard-filter-indicators-container:hover .color-bar.badge-20,.filter-badge.badge-20{background-color:#087f5b}.active .color-bar.badge-19,.dashboard-component-chart-holder:hover .color-bar.badge-19,.dashboard-filter-indicators-container:hover .color-bar.badge-19,.filter-badge.badge-19{background-color:#5c940d}.active .color-bar.badge-18,.dashboard-component-chart-holder:hover .color-bar.badge-18,.dashboard-filter-indicators-container:hover .color-bar.badge-18,.filter-badge.badge-18{background-color:#c92a2a}.active .color-bar.badge-17,.dashboard-component-chart-holder:hover .color-bar.badge-17,.dashboard-filter-indicators-container:hover .color-bar.badge-17,.filter-badge.badge-17{background-color:#5f3dc4}.active .color-bar.badge-16,.dashboard-component-chart-holder:hover .color-bar.badge-16,.dashboard-filter-indicators-container:hover .color-bar.badge-16,.filter-badge.badge-16{background-color:#0b7285}.active .color-bar.badge-15,.dashboard-component-chart-holder:hover .color-bar.badge-15,.dashboard-filter-indicators-container:hover .color-bar.badge-15,.filter-badge.badge-15{background-color:#a61e4d}.active .color-bar.badge-14,.dashboard-component-chart-holder:hover .color-bar.badge-14,.dashboard-filter-indicators-container:hover .color-bar.badge-14,.filter-badge.badge-14{background-color:#d9480f}.active .color-bar.badge-13,.dashboard-component-chart-holder:hover .color-bar.badge-13,.dashboard-filter-indicators-container:hover .color-bar.badge-13,.filter-badge.badge-13{background-color:#e67700}.active .color-bar.badge-12,.dashboard-component-chart-holder:hover .color-bar.badge-12,.dashboard-filter-indicators-container:hover .color-bar.badge-12,.filter-badge.badge-12{background-color:#2b8a3e}.active .color-bar.badge-11,.dashboard-component-chart-holder:hover .color-bar.badge-11,.dashboard-filter-indicators-container:hover .color-bar.badge-11,.filter-badge.badge-11{background-color:#1864ab}.active .color-bar.badge-10,.dashboard-component-chart-holder:hover .color-bar.badge-10,.dashboard-filter-indicators-container:hover .color-bar.badge-10,.filter-badge.badge-10{background-color:#12b886}.active .color-bar.badge-9,.dashboard-component-chart-holder:hover .color-bar.badge-9,.dashboard-filter-indicators-container:hover .color-bar.badge-9,.filter-badge.badge-9{background-color:#74b816}.active .color-bar.badge-8,.dashboard-component-chart-holder:hover .color-bar.badge-8,.dashboard-filter-indicators-container:hover .color-bar.badge-8,.filter-badge.badge-8{background-color:#fa5252}.active .color-bar.badge-7,.dashboard-component-chart-holder:hover .color-bar.badge-7,.dashboard-filter-indicators-container:hover .color-bar.badge-7,.filter-badge.badge-7{background-color:#7950f2}.active .color-bar.badge-6,.dashboard-component-chart-holder:hover .color-bar.badge-6,.dashboard-filter-indicators-container:hover .color-bar.badge-6,.filter-badge.badge-6{background-color:#15aabf}.active .color-bar.badge-5,.dashboard-component-chart-holder:hover .color-bar.badge-5,.dashboard-filter-indicators-container:hover .color-bar.badge-5,.filter-badge.badge-5{background-color:#e64980}.active .color-bar.badge-4,.dashboard-component-chart-holder:hover .color-bar.badge-4,.dashboard-filter-indicators-container:hover .color-bar.badge-4,.filter-badge.badge-4{background-color:#f76707}.active .color-bar.badge-3,.dashboard-component-chart-holder:hover .color-bar.badge-3,.dashboard-filter-indicators-container:hover .color-bar.badge-3,.filter-badge.badge-3{background-color:#fab005}.active .color-bar.badge-2,.dashboard-component-chart-holder:hover .color-bar.badge-2,.dashboard-filter-indicators-container:hover .color-bar.badge-2,.filter-badge.badge-2{background-color:#40c057}.active .color-bar.badge-1,.dashboard-component-chart-holder:hover .color-bar.badge-1,.dashboard-filter-indicators-container:hover .color-bar.badge-1,.filter-badge.badge-1{background-color:#228be6}.tab-pane{min-height:400px;background:#fff;border:1px solid #cfd8dc;border-top:0}.label{display:inline-block;margin-right:5px;margin-bottom:5px}