/*!
 * https://github.com/YouCanBookMe/react-datetime
 */.rdt{position:relative}.rdtPicker{display:none;position:absolute;width:250px;padding:4px;margin-top:1px;z-index:99999!important;background:#fff;box-shadow:0 1px 3px rgba(0,0,0,.1);border:1px solid #f9f9f9}.rdtOpen .rdtPicker{display:block}.rdtStatic .rdtPicker{box-shadow:none;position:static}.rdtPicker .rdtTimeToggle{text-align:center}.rdtPicker table{width:100%;margin:0}.rdtPicker td,.rdtPicker th{text-align:center;height:28px}.rdtPicker td{cursor:pointer}.rdtPicker .rdtTimeToggle:hover,.rdtPicker td.rdtDay:hover,.rdtPicker td.rdtHour:hover,.rdtPicker td.rdtMinute:hover,.rdtPicker td.rdtSecond:hover{background:#eee;cursor:pointer}.rdtPicker td.rdtNew,.rdtPicker td.rdtOld{color:#999}.rdtPicker td.rdtToday{position:relative}.rdtPicker td.rdtToday:before{content:"";display:inline-block;border-left:7px solid transparent;border-bottom:7px solid #428bca;border-top-color:rgba(0,0,0,.2);position:absolute;bottom:4px;right:4px}.rdtPicker td.rdtActive,.rdtPicker td.rdtActive:hover{background-color:#428bca;color:#fff;text-shadow:0 -1px 0 rgba(0,0,0,.25)}.rdtPicker td.rdtActive.rdtToday:before{border-bottom-color:#fff}.rdtPicker td.rdtDisabled,.rdtPicker td.rdtDisabled:hover{background:none;color:#999;cursor:not-allowed}.rdtPicker td span.rdtOld{color:#999}.rdtPicker td span.rdtDisabled,.rdtPicker td span.rdtDisabled:hover{background:none;color:#999;cursor:not-allowed}.rdtPicker th{border-bottom:1px solid #f9f9f9}.rdtPicker .dow{width:14.2857%;border-bottom:none;cursor:default}.rdtPicker th.rdtSwitch{width:100px}.rdtPicker th.rdtNext,.rdtPicker th.rdtPrev{font-size:21px;vertical-align:top}.rdtNext span,.rdtPrev span{display:block;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.rdtPicker th.rdtDisabled,.rdtPicker th.rdtDisabled:hover{background:none;color:#999;cursor:not-allowed}.rdtPicker thead tr:first-child th{cursor:pointer}.rdtPicker thead tr:first-child th:hover{background:#eee}.rdtPicker tfoot{border-top:1px solid #f9f9f9}.rdtPicker button{border:none;background:none;cursor:pointer}.rdtPicker button:hover{background-color:#eee}.rdtPicker thead button{width:100%;height:100%}td.rdtMonth,td.rdtYear{height:50px;width:25%;cursor:pointer}td.rdtMonth:hover,td.rdtYear:hover{background:#eee}.rdtCounters{display:inline-block}.rdtCounters>div{float:left}.rdtCounter{height:100px;width:40px}.rdtCounterSeparator{line-height:100px}.rdtCounter .rdtBtn{height:40%;line-height:40px;cursor:pointer;display:block;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.rdtCounter .rdtBtn:hover{background:#eee}.rdtCounter .rdtCount{height:20%;font-size:1.2em}.rdtMilli{vertical-align:middle;padding-left:8px;width:48px}.rdtMilli input{width:100%;font-size:1.2em;margin-top:37px}.rdtTime td{cursor:default}.rdtPicker table{font-size:12px}.Control{padding-bottom:4px}.optionWrapper{overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.CollectionControl .list-group-item i.fa{padding-top:5px}.RefreshLabel{color:#b3b3b3}.RefreshLabel:hover{color:#20a7c9}.TableSelector .fa-refresh{padding-left:9px}.TableSelector .refresh-col{display:flex;align-items:center;width:30px}.TableSelector .section{padding-bottom:5px;display:flex;flex-direction:row}.TableSelector .select{flex-grow:1}.TableSelector .divider{border-bottom:1px solid #f5f5f5;margin:15px 0}.TableLabel{white-space:nowrap}.CRUD .text-right{text-align:right}.CRUD .empty-collection{padding:10px}.CRUD .control-label{font-weight:700;font-size:16px}.CRUD .tiny-cell,.CRUD i.fa-caret-down,.CRUD i.fa-caret-up{width:5px}.CRUD td.expanded{border-top:0;padding:0}.CRUD .frame{border:1px solid #a3a3a3;border-radius:4px;padding:10;background:#f5f5f5}.Datasource .tab-content{height:600px;overflow:auto}.Datasource .change-warning{margin:16px 10px 0;color:#ff7f44}.Datasource .change-warning .bold{font-weight:700}#datasource_menu{border-radius:4px;padding-left:8px;padding-right:8px}#datasource_menu .caret{position:relative;padding-right:8px;margin-left:4px;color:#fff;top:-8px}#datasource_menu+ul{margin-top:26px}

/*! =======================================================
                      VERSION  10.4.0              
========================================================= */
/*! =========================================================
 * bootstrap-slider.js
 *
 * Maintainers:
 *		Kyle Kemp
 *			- Twitter: @seiyria
 *			- Github:  seiyria
 *		Rohit Kalkur
 *			- Twitter: @Rovolutionary
 *			- Github:  rovolution
 *
 * =========================================================
  *
 * bootstrap-slider is released under the MIT License
 * Copyright (c) 2017 Kyle Kemp, Rohit Kalkur, and contributors
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * ========================================================= */.slider{display:inline-block;vertical-align:middle;position:relative}.slider.slider-horizontal{width:210px;height:20px}.slider.slider-horizontal .slider-track{height:10px;width:100%;margin-top:-5px;top:50%;left:0}.slider.slider-horizontal .slider-selection,.slider.slider-horizontal .slider-track-high,.slider.slider-horizontal .slider-track-low{height:100%;top:0;bottom:0}.slider.slider-horizontal .slider-handle,.slider.slider-horizontal .slider-tick{margin-left:-10px}.slider.slider-horizontal .slider-handle.triangle,.slider.slider-horizontal .slider-tick.triangle{position:relative;top:50%;-ms-transform:translateY(-50%);transform:translateY(-50%);border-width:0 10px 10px;width:0;height:0;border-bottom-color:#2e6da4;margin-top:0}.slider.slider-horizontal .slider-tick-container{white-space:nowrap;position:absolute;top:0;left:0;width:100%}.slider.slider-horizontal .slider-tick-label-container{white-space:nowrap;margin-top:20px}.slider.slider-horizontal .slider-tick-label-container .slider-tick-label{padding-top:4px;display:inline-block;text-align:center}.slider.slider-horizontal .tooltip{-ms-transform:translateX(-50%);transform:translateX(-50%)}.slider.slider-horizontal.slider-rtl .slider-track{left:auto;right:0}.slider.slider-horizontal.slider-rtl .slider-handle,.slider.slider-horizontal.slider-rtl .slider-tick{margin-left:0;margin-right:-10px}.slider.slider-horizontal.slider-rtl .slider-tick-container{left:auto;right:0}.slider.slider-horizontal.slider-rtl .tooltip{-ms-transform:translateX(50%);transform:translateX(50%)}.slider.slider-vertical{height:210px;width:20px}.slider.slider-vertical .slider-track{width:10px;height:100%;left:25%;top:0}.slider.slider-vertical .slider-selection{width:100%;left:0;top:0;bottom:0}.slider.slider-vertical .slider-track-high,.slider.slider-vertical .slider-track-low{width:100%;left:0;right:0}.slider.slider-vertical .slider-handle,.slider.slider-vertical .slider-tick{margin-top:-10px}.slider.slider-vertical .slider-handle.triangle,.slider.slider-vertical .slider-tick.triangle{border-width:10px 0 10px 10px;width:1px;height:1px;border-left-color:#2e6da4;border-right-color:#2e6da4;margin-left:0;margin-right:0}.slider.slider-vertical .slider-tick-label-container{white-space:nowrap}.slider.slider-vertical .slider-tick-label-container .slider-tick-label{padding-left:4px}.slider.slider-vertical .tooltip{-ms-transform:translateY(-50%);transform:translateY(-50%)}.slider.slider-vertical.slider-rtl .slider-track{left:auto;right:25%}.slider.slider-vertical.slider-rtl .slider-selection{left:auto;right:0}.slider.slider-vertical.slider-rtl .slider-handle.triangle,.slider.slider-vertical.slider-rtl .slider-tick.triangle{border-width:10px 10px 10px 0}.slider.slider-vertical.slider-rtl .slider-tick-label-container .slider-tick-label{padding-left:0;padding-right:4px}.slider.slider-disabled .slider-handle{background-image:-webkit-linear-gradient(top,#dfdfdf,#bebebe);background-image:-o-linear-gradient(top,#dfdfdf 0,#bebebe 100%);background-image:linear-gradient(180deg,#dfdfdf 0,#bebebe);background-repeat:repeat-x;filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#ffdfdfdf",endColorstr="#ffbebebe",GradientType=0)}.slider.slider-disabled .slider-track{background-image:-webkit-linear-gradient(top,#e5e5e5,#e9e9e9);background-image:-o-linear-gradient(top,#e5e5e5 0,#e9e9e9 100%);background-image:linear-gradient(180deg,#e5e5e5 0,#e9e9e9);background-repeat:repeat-x;filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#ffe5e5e5",endColorstr="#ffe9e9e9",GradientType=0);cursor:not-allowed}.slider input{display:none}.slider .tooltip{pointer-events:none}.slider .tooltip.top{margin-top:-36px}.slider .tooltip-inner{white-space:nowrap;max-width:none}.slider .hide{display:none}.slider-track{cursor:pointer;background-image:-webkit-linear-gradient(top,#f5f5f5,#f9f9f9);background-image:-o-linear-gradient(top,#f5f5f5 0,#f9f9f9 100%);background-image:linear-gradient(180deg,#f5f5f5 0,#f9f9f9);filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#fff5f5f5",endColorstr="#fff9f9f9",GradientType=0);-webkit-box-shadow:inset 0 1px 2px rgba(0,0,0,.1);box-shadow:inset 0 1px 2px rgba(0,0,0,.1)}.slider-selection,.slider-track{position:absolute;background-repeat:repeat-x;border-radius:4px}.slider-selection{background-image:-webkit-linear-gradient(top,#f9f9f9,#f5f5f5);background-image:-o-linear-gradient(top,#f9f9f9 0,#f5f5f5 100%);background-image:linear-gradient(180deg,#f9f9f9 0,#f5f5f5);filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#fff9f9f9",endColorstr="#fff5f5f5",GradientType=0);-webkit-box-shadow:inset 0 -1px 0 rgba(0,0,0,.15);box-shadow:inset 0 -1px 0 rgba(0,0,0,.15);-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box}.slider-selection.tick-slider-selection{background-image:-webkit-linear-gradient(top,#8ac1ef,#82b3de);background-image:-o-linear-gradient(top,#8ac1ef 0,#82b3de 100%);background-image:linear-gradient(180deg,#8ac1ef 0,#82b3de);background-repeat:repeat-x;filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#ff8ac1ef",endColorstr="#ff82b3de",GradientType=0)}.slider-track-high,.slider-track-low{position:absolute;background:transparent;-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box;border-radius:4px}.slider-handle{position:absolute;top:0;width:20px;height:20px;background-color:#337ab7;background-image:-webkit-linear-gradient(top,#337ab7,#2e6da4);background-image:-o-linear-gradient(top,#337ab7 0,#2e6da4 100%);background-image:linear-gradient(180deg,#337ab7 0,#2e6da4);background-repeat:repeat-x;filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#ff337ab7",endColorstr="#ff2e6da4",GradientType=0);filter:none;-webkit-box-shadow:inset 0 1px 0 hsla(0,0%,100%,.2),0 1px 2px rgba(0,0,0,.05);box-shadow:inset 0 1px 0 hsla(0,0%,100%,.2),0 1px 2px rgba(0,0,0,.05);border:0 solid transparent}.slider-handle:hover{cursor:pointer}.slider-handle.round{border-radius:50%}.slider-handle.custom,.slider-handle.triangle{background:transparent none}.slider-handle.custom:before{line-height:20px;font-size:20px;content:"\2605";color:#726204}.slider-tick{position:absolute;cursor:pointer;width:20px;height:20px;background-image:-webkit-linear-gradient(top,#f9f9f9,#f5f5f5);background-image:-o-linear-gradient(top,#f9f9f9 0,#f5f5f5 100%);background-image:linear-gradient(180deg,#f9f9f9 0,#f5f5f5);background-repeat:repeat-x;filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#fff9f9f9",endColorstr="#fff5f5f5",GradientType=0);-webkit-box-shadow:inset 0 -1px 0 rgba(0,0,0,.15);box-shadow:inset 0 -1px 0 rgba(0,0,0,.15);-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box;filter:none;opacity:.8;border:0 solid transparent}.slider-tick.round{border-radius:50%}.slider-tick.custom,.slider-tick.triangle{background:transparent none}.slider-tick.custom:before{line-height:20px;font-size:20px;content:"\2605";color:#726204}.slider-tick.in-selection{background-image:-webkit-linear-gradient(top,#8ac1ef,#82b3de);background-image:-o-linear-gradient(top,#8ac1ef 0,#82b3de 100%);background-image:linear-gradient(180deg,#8ac1ef 0,#82b3de);background-repeat:repeat-x;filter:progid:DXImageTransform.Microsoft.gradient(startColorstr="#ff8ac1ef",endColorstr="#ff82b3de",GradientType=0);opacity:1}.BootstrapSliderWrapper .slider-selection{background:#f5f5f5}.BootstrapSliderWrapper .slider-handle{background:#cfd8dc}.viztype-label{margin-top:10px;text-align:center;font-size:14px}.viztype-selector-container{cursor:pointer;margin-top:10px;margin-bottom:10px}.viztype-selector-container:hover img{border:1px solid #a3a3a3}.viztype-selector-container.selected{cursor:not-allowed;opacity:1}.viztype-selector-container.selected img{border:1px solid #263238}.viztype-selector-container img{border:1px solid #cfd8dc;border-radius:4px;transition:border-color .3s}.viztype-control-search-box{margin-bottom:10px}.filter-badge{width:20px;height:20px;background-color:#cfd8dc;z-index:10;border-radius:4px}.filter-badge path{fill:#fff}.filter-badge.badge-group,.filter-badge .color-bar.badge-group{background-color:#333}.dashboard .filter_box .slice_container>div:not(.alert){padding-top:0}.filter_box{padding:10px 0;overflow:visible!important}.filter_box:hover{z-index:3000}.m-b-5{margin-bottom:5px}.filter-container{display:flex}.filter-container label{display:flex;font-weight:700;margin:0 0 8px 8px}.filter-container .filter-badge-container{width:30px;padding-right:10px}.filter-container .filter-badge-container+div{width:100%}.nv-distx,.nv-disty,.nv-noninteractive,.nvd3 .nv-axis,.nvd3.nv-pie .nv-label,.nvd3.nv-sparklineplus g.nv-hoverValue{pointer-events:none}.nvtooltip h3,.nvtooltip p{margin:0;text-align:center}.nvd3 .nv-axis{opacity:1}.nvd3 .nv-axis.nv-disabled,.nvd3 .nv-controlsWrap .nv-legend .nv-check-box .nv-check{opacity:0}.nvd3 .nv-axis path{fill:none;stroke:#000;stroke-opacity:.75;shape-rendering:crispEdges}.nvd3 .nv-axis path.domain{stroke-opacity:.75}.nvd3 .nv-axis.nv-x path.domain{stroke-opacity:0}.nvd3 .nv-axis line{fill:none;stroke:#e5e5e5;shape-rendering:crispEdges}.nvd3 .nv-axis .zero line,.nvd3 .nv-axis line.zero{stroke-opacity:.75}.nvd3 .nv-axis .nv-axisMaxMin text{font-weight:700}.nvd3 .x2 .nv-axis .nv-axisMaxMin text,.nvd3 .x3 .nv-axis .nv-axisMaxMin text,.nvd3 .x .nv-axis .nv-axisMaxMin text{text-anchor:middle}.nvd3 .nv-bars rect{fill-opacity:.75;transition:fill-opacity .25s linear}.nvd3 .nv-bars rect.hover{fill-opacity:1}.nvd3 .nv-bars .hover rect{fill:#add8e6}.nvd3 .nv-bars text{fill:transparent}.nvd3 .nv-bars .hover text{fill:#000}.nvd3 .nv-discretebar .nv-groups rect,.nvd3 .nv-multibar .nv-groups rect,.nvd3 .nv-multibarHorizontal .nv-groups rect{stroke-opacity:0;transition:fill-opacity .25s linear}.nvd3 .nv-candlestickBar .nv-ticks rect:hover,.nvd3 .nv-discretebar .nv-groups rect:hover,.nvd3 .nv-multibar .nv-groups rect:hover,.nvd3 .nv-multibarHorizontal .nv-groups rect:hover{fill-opacity:1}.nvd3 .nv-discretebar .nv-groups text,.nvd3 .nv-multibarHorizontal .nv-groups text{font-weight:700;fill:#000;stroke:transparent}.nvd3 .nv-boxplot circle{fill-opacity:.5}.nvd3 .nv-boxplot circle:hover,.nvd3 .nv-boxplot rect:hover{fill-opacity:1}.nvd3 line.nv-boxplot-median{stroke:#000}.nv-boxplot-tick:hover{stroke-width:2.5px}.nvd3.nv-bullet{font:10px sans-serif}.nvd3.nv-bullet .nv-measure{fill-opacity:.8}.nvd3.nv-bullet .nv-measure:hover{fill-opacity:1}.nvd3.nv-bullet .nv-marker{stroke:#000;stroke-width:2px}.nvd3.nv-bullet .nv-markerTriangle{stroke:#000;fill:#fff;stroke-width:1.5px}.nvd3.nv-bullet .nv-markerLine{stroke:#000;stroke-width:1.5px}.nvd3.nv-bullet .nv-tick line{stroke:#666;stroke-width:.5px}.nvd3.nv-bullet .nv-range.nv-s0{fill:#eee}.nvd3.nv-bullet .nv-range.nv-s1{fill:#ddd}.nvd3.nv-bullet .nv-range.nv-s2{fill:#ccc}.nvd3.nv-bullet .nv-title{font-size:14px;font-weight:700}.nvd3.nv-bullet .nv-subtitle{fill:#999}.nvd3.nv-bullet .nv-range{fill:#bababa;fill-opacity:.4}.nvd3.nv-bullet .nv-range:hover{fill-opacity:.7}.nvd3.nv-candlestickBar .nv-ticks .nv-tick{stroke-width:1px}.nvd3.nv-candlestickBar .nv-ticks .nv-tick.hover{stroke-width:2px}.nvd3.nv-candlestickBar .nv-ticks .nv-tick.positive rect{stroke:#2ca02c;fill:#2ca02c}.nvd3.nv-candlestickBar .nv-ticks .nv-tick.negative rect{stroke:#d62728;fill:#d62728}.with-transitions .nv-candlestickBar .nv-ticks .nv-tick{transition:stroke-width .25s linear,stroke-opacity .25s linear}.nvd3.nv-candlestickBar .nv-ticks line{stroke:#333}.nv-force-node{stroke:#fff;stroke-width:1.5px}.nv-force-link{stroke:#999;stroke-opacity:.6}.nv-force-node text{stroke-width:0}.nvd3 .nv-check-box .nv-box{fill-opacity:0;stroke-width:2}.nvd3 .nv-check-box .nv-check{fill-opacity:0;stroke-width:4}.nvd3 .nv-series.nv-disabled .nv-check-box .nv-check{fill-opacity:0;stroke-opacity:0}.nvd3.nv-linePlusBar .nv-bar rect{fill-opacity:.75}.nvd3.nv-linePlusBar .nv-bar rect:hover{fill-opacity:1}.nvd3 .nv-groups path.nv-line{fill:none}.nvd3 .nv-groups path.nv-area{stroke:none}.nvd3.nv-line .nvd3.nv-scatter .nv-groups .nv-point{fill-opacity:0;stroke-opacity:0}.nvd3.nv-scatter.nv-single-point .nv-groups .nv-point{fill-opacity:.5!important;stroke-opacity:.5!important}.with-transitions .nvd3 .nv-groups .nv-point{transition:stroke-width .25s linear,stroke-opacity .25s linear}.nvd3 .nv-groups .nv-point.hover,.nvd3.nv-scatter .nv-groups .nv-point.hover{stroke-width:7px;fill-opacity:.95!important;stroke-opacity:.95!important}.nvd3 .nv-point-paths path{stroke:#aaa;stroke-opacity:0;fill:#eee;fill-opacity:0}.nvd3 .nv-indexLine{cursor:ew-resize}svg.nvd3-svg{-webkit-user-select:none;-ms-user-select:none;user-select:none;display:block;width:100%;height:100%}.nvtooltip.with-3d-shadow,.with-3d-shadow .nvtooltip{box-shadow:0 5px 10px rgba(0,0,0,.2);border-radius:5px}.nvd3 text{font:400 12px Arial,sans-serif}.nvd3 .title{font:700 14px Arial,sans-serif}.nvd3 .nv-background{fill:#fff;fill-opacity:0}.nvd3.nv-noData{font-size:18px;font-weight:700}.nv-brush .extent{fill-opacity:.125;shape-rendering:crispEdges}.nv-brush .resize path{fill:#eee;stroke:#666}.nvd3 .nv-legend .nv-series{cursor:pointer}.nvd3 .nv-legend .nv-disabled circle{fill-opacity:0}.nvd3 .nv-brush .extent{fill-opacity:0!important}.nvd3 .nv-brushBackground rect{stroke:#000;stroke-width:.4;fill:#fff;fill-opacity:.7}@media print{.nvd3 text{stroke-width:0;fill-opacity:1}}.nvd3.nv-ohlcBar .nv-ticks .nv-tick{stroke-width:1px}.nvd3.nv-ohlcBar .nv-ticks .nv-tick.hover{stroke-width:2px}.nvd3.nv-ohlcBar .nv-ticks .nv-tick.positive{stroke:#2ca02c}.nvd3.nv-ohlcBar .nv-ticks .nv-tick.negative{stroke:#d62728}.nvd3 .background path{fill:none;stroke:#eee;stroke-opacity:.4;shape-rendering:crispEdges}.nvd3 .foreground path{fill:none;stroke-opacity:.7}.nvd3 .nv-parallelCoordinates-brush .extent{fill:#fff;fill-opacity:.6;stroke:grey;shape-rendering:crispEdges}.nvd3 .nv-parallelCoordinates .hover{fill-opacity:1;stroke-width:3px}.nvd3 .missingValuesline line{fill:none;stroke:#000;stroke-width:1;stroke-opacity:1;stroke-dasharray:5,5}.nvd3.nv-pie .nv-pie-title{font-size:24px;fill:rgba(19,196,249,.59)}.nvd3.nv-pie .nv-slice text{stroke:#000;stroke-width:0}.nvd3.nv-pie path{transition:fill-opacity .25s linear,stroke-width .25s linear,stroke-opacity .25s linear;stroke:#fff;stroke-width:1px;stroke-opacity:1;fill-opacity:.7}.nvd3.nv-pie .hover path{fill-opacity:1}.nvd3.nv-pie .nv-label rect{fill-opacity:0;stroke-opacity:0}.nvd3 .nv-groups .nv-point.hover{stroke-width:20px;stroke-opacity:.5}.nvd3 .nv-scatter .nv-point.hover{fill-opacity:1}.nvd3.nv-sparkline path{fill:none}.nvd3.nv-sparklineplus .nv-hoverValue line{stroke:#333;stroke-width:1.5px}.nvd3.nv-sparklineplus,.nvd3.nv-sparklineplus g{pointer-events:all}.nvd3 .nv-interactiveGuideLine,.nvtooltip,.nvtooltip-pending-removal{pointer-events:none}.nvd3 .nv-hoverArea{fill-opacity:0;stroke-opacity:0}.nvd3.nv-sparklineplus .nv-xValue,.nvd3.nv-sparklineplus .nv-yValue{stroke-width:0;font-size:.9em;font-weight:400}.nvd3.nv-sparklineplus .nv-yValue{stroke:#f66}.nvd3.nv-sparklineplus .nv-maxValue{stroke:#2ca02c;fill:#2ca02c}.nvd3.nv-sparklineplus .nv-minValue{stroke:#d62728;fill:#d62728}.nvd3.nv-sparklineplus .nv-currentValue{font-weight:700;font-size:1.1em}.nvd3.nv-stackedarea path.nv-area{fill-opacity:.7;stroke-opacity:0;transition:fill-opacity .25s linear,stroke-opacity .25s linear}.nvd3.nv-stackedarea path.nv-area.hover{fill-opacity:.9}.nvd3.nv-stackedarea .nv-groups .nv-point{stroke-opacity:0;fill-opacity:0}.nvtooltip{position:absolute;color:#000;padding:1px;z-index:10000;display:block;font-family:Arial,sans-serif;font-size:13px;text-align:left;white-space:nowrap;-webkit-user-select:none;-ms-user-select:none;user-select:none;background:hsla(0,0%,100%,.8);border:1px solid rgba(0,0,0,.5);border-radius:4px}.nvtooltip.with-transitions,.with-transitions .nvtooltip{transition:opacity 50ms linear;transition-delay:.2s}.nvtooltip.x-nvtooltip,.nvtooltip.y-nvtooltip{padding:8px}.nvtooltip h3{padding:4px 14px;line-height:18px;font-weight:400;background-color:hsla(0,0%,96.9%,.75);color:#000;border-bottom:1px solid #ebebeb;border-radius:5px 5px 0 0}.nvtooltip p{padding:5px 14px}.nvtooltip span{display:inline-block;margin:2px 0}.nvtooltip table{margin:6px;border-spacing:0}.nvtooltip table td{padding:2px 9px 2px 0;vertical-align:middle}.nvtooltip table td.key{font-weight:400}.nvtooltip table td.key.total{font-weight:700}.nvtooltip table td.value{text-align:right;font-weight:700}.nvtooltip table td.percent{color:#a9a9a9}.nvtooltip table tr.highlight td{padding:1px 9px 1px 0;border-bottom-style:solid;border-bottom-width:1px;border-top-style:solid;border-top-width:1px}.nvtooltip table td.legend-color-guide div{vertical-align:middle;width:12px;height:12px;border:1px solid #999}.nvtooltip .footer{padding:3px;text-align:center}.nvtooltip-pending-removal{display:none}.nvd3 line.nv-guideline{stroke:#ccc}.superset-legacy-chart-nvd3-dist-bar{overflow-x:auto!important}.superset-legacy-chart-nvd3-dist-bar svg.nvd3-svg{width:auto;font-size:14px}.superset-legacy-chart-nvd3-bar{overflow-x:auto!important}.superset-legacy-chart-nvd3-bar svg.nvd3-svg{width:auto}.superset-legacy-chart-nvd3 .nv-x text{font-size:12px}.superset-legacy-chart-nvd3 g.superset path{stroke-dasharray:5,5}.superset-legacy-chart-nvd3 .nvtooltip tr.highlight td{font-weight:700;font-size:15px!important}.superset-legacy-chart-nvd3 text.nv-axislabel{font-size:14px;font-size:14px!important}.superset-legacy-chart-nvd3 g.solid path,.superset-legacy-chart-nvd3 line.solid{stroke-dasharray:unset}.superset-legacy-chart-nvd3 g.dashed path,.superset-legacy-chart-nvd3 line.dashed{stroke-dasharray:5,5}.superset-legacy-chart-nvd3 g.longDashed path,.superset-legacy-chart-nvd3 line.longDashed{stroke-dasharray:10,2}.superset-legacy-chart-nvd3 g.dotted path,.superset-legacy-chart-nvd3 line.dotted{stroke-dasharray:1,1}.superset-legacy-chart-nvd3 g.opacityLow path,.superset-legacy-chart-nvd3 line.opacityLow{stroke-opacity:.2}.superset-legacy-chart-nvd3 g.opacityMedium path,.superset-legacy-chart-nvd3 line.opacityMedium{stroke-opacity:.5}.superset-legacy-chart-nvd3 g.opacityHigh path,.superset-legacy-chart-nvd3 line.opacityHigh{stroke-opacity:.8}.superset-legacy-chart-nvd3 g.time-shift-0 path,.superset-legacy-chart-nvd3 line.time-shift-0{stroke-dasharray:5,5}.superset-legacy-chart-nvd3 g.time-shift-1 path,.superset-legacy-chart-nvd3 line.time-shift-1{stroke-dasharray:1,5}.superset-legacy-chart-nvd3 g.time-shift-2 path,.superset-legacy-chart-nvd3 line.time-shift-2{stroke-dasharray:10,5}.superset-legacy-chart-nvd3 g.time-shift-3 path,.superset-legacy-chart-nvd3 line.time-shift-3{stroke-dasharray:5,1}.superset-legacy-chart-nvd3 g.time-shift-4 path,.superset-legacy-chart-nvd3 line.time-shift-4{stroke-dasharray:5,10}.superset-legacy-chart-nvd3 g.time-shift-5 path,.superset-legacy-chart-nvd3 line.time-shift-5{stroke-dasharray:.9}.superset-legacy-chart-nvd3 g.time-shift-6 path,.superset-legacy-chart-nvd3 line.time-shift-6{stroke-dasharray:15,10,5}.superset-legacy-chart-nvd3 g.time-shift-7 path,.superset-legacy-chart-nvd3 line.time-shift-7{stroke-dasharray:15,10,5,10}.superset-legacy-chart-nvd3 g.time-shift-8 path,.superset-legacy-chart-nvd3 line.time-shift-8{stroke-dasharray:15,10,5,10,15}.superset-legacy-chart-nvd3 g.time-shift-9 path,.superset-legacy-chart-nvd3 line.time-shift-9{stroke-dasharray:5,5,1,5}.superset-legacy-chart-nvd3-tr-highlight{border-top:1px solid;border-bottom:1px solid;font-weight:700}.superset-legacy-chart-nvd3-tr-total{font-weight:700}.nvtooltip .tooltip-header{white-space:nowrap;font-weight:700}.nvtooltip tbody tr:not(.tooltip-header) td:nth-child(2){word-break:break-word}.superset-legacy-chart-nvd3 .nv-noData.body{font-size:14px;font-weight:400}.d3-tip.nv-event-annotation-layer-NATIVE,.d3-tip.nv-event-annotation-layer-table{width:200px;border-radius:2px;background-color:#484848;fill-opacity:.6;margin:8px;padding:8px;color:#fff}.d3-tip.nv-event-annotation-layer-NATIVE:after,.d3-tip.nv-event-annotation-layer-table:after{content:"\25BC";font-size:14px;color:#484848;position:absolute;bottom:-14px;left:94px}.cal-heatmap-container{display:block}.cal-heatmap-container .graph-label{fill:#999;font-size:10px}.cal-heatmap-container .graph,.cal-heatmap-container .graph-legend rect{shape-rendering:crispedges}.cal-heatmap-container .graph-rect{fill:#ededed}.cal-heatmap-container .graph-subdomain-group rect:hover{stroke:#000;stroke-width:1px}.cal-heatmap-container .subdomain-text{font-size:8px;fill:#999;pointer-events:none}.cal-heatmap-container .hover_cursor:hover{cursor:pointer}.cal-heatmap-container .qi{background-color:#999;fill:#999}.cal-heatmap-container .q1{background-color:#dae289;fill:#dae289}.cal-heatmap-container .q2{background-color:#cedb9c;fill:#9cc069}.cal-heatmap-container .q3{background-color:#b5cf6b;fill:#669d45}.cal-heatmap-container .q4{background-color:#637939;fill:#637939}.cal-heatmap-container .q5{background-color:#3b6427;fill:#3b6427}.cal-heatmap-container rect.highlight{stroke:#444;stroke-width:1}.cal-heatmap-container text.highlight{fill:#444}.cal-heatmap-container rect.highlight-now{stroke:red}.cal-heatmap-container text.highlight-now{fill:red;font-weight:800}.cal-heatmap-container .domain-background{fill:none;shape-rendering:crispedges}.ch-tooltip{padding:10px;background:#222;color:#bbb;font-size:12px;line-height:1.4;width:140px;position:absolute;z-index:99999;text-align:center;border-radius:2px;box-shadow:2px 2px 2px rgba(0,0,0,.2);display:none;box-sizing:border-box}.ch-tooltip:after{position:absolute;width:0;height:0;content:"";padding:0;display:block;bottom:-6px;left:50%;margin-left:-6px;border-color:#222 transparent transparent;border-style:solid;border-width:6px 6px 0}.superset-legacy-chart-country-map svg{background-color:#feffff}.superset-legacy-chart-country-map{position:relative}.superset-legacy-chart-country-map .background{fill:hsla(0,0%,100%,0);pointer-events:all}.superset-legacy-chart-country-map .map-layer{fill:#fff;stroke:#aaa}.superset-legacy-chart-country-map .effect-layer{pointer-events:none}.superset-legacy-chart-country-map .text-layer{color:#333;text-anchor:middle;pointer-events:none}.superset-legacy-chart-country-map text.result-text{font-weight:300;font-size:24px}.superset-legacy-chart-country-map text.big-text{font-weight:700;font-size:16px}.superset-legacy-chart-country-map path.region{cursor:pointer;stroke:#eee}.superset-legacy-chart-force-directed path.link{fill:none;stroke:#000;stroke-width:1.5px}.superset-legacy-chart-force-directed circle{fill:#ccc;stroke:#000;stroke-width:1.5px;stroke-opacity:1;opacity:.75}.superset-legacy-chart-force-directed text{fill:#000;font:10px sans-serif;pointer-events:none}.d3-tip{line-height:1;padding:12px;background:rgba(0,0,0,.8);color:#fff;border-radius:2px;pointer-events:none;z-index:1000}.d3-tip:after{box-sizing:border-box;display:inline;font-size:10px;width:100%;line-height:1;color:rgba(0,0,0,.8);position:absolute;pointer-events:none}.d3-tip.n:after{content:"\25BC";margin:-1px 0 0;top:100%;left:0;text-align:center}.d3-tip.e:after{content:"\25C0";margin:-4px 0 0;top:50%;left:-8px}.d3-tip.s:after{content:"\25B2";margin:0 0 1px;top:-8px;left:0;text-align:center}.d3-tip.w:after{content:"\25B6";margin:-4px 0 0 -1px;top:50%;left:100%}.superset-legacy-chart-heatmap{position:relative;top:0;left:0;height:100%}.superset-legacy-chart-heatmap .axis text{font:12px sans-serif;text-rendering:optimizeLegibility;fill:#555}.superset-legacy-chart-heatmap .background-rect{stroke:#ddd;fill-opacity:0;pointer-events:all}.superset-legacy-chart-heatmap .axis line,.superset-legacy-chart-heatmap .axis path{fill:none;stroke:#ddd;shape-rendering:crispEdges}.superset-legacy-chart-heatmap canvas,.superset-legacy-chart-heatmap img{image-rendering:optimizeSpeed;image-rendering:-moz-crisp-edges;image-rendering:-webkit-optimize-contrast;image-rendering:-o-crisp-edges;image-rendering:pixelated;-ms-interpolation-mode:nearest-neighbor}.superset-legacy-chart-heatmap .legendCells text{font-size:10px;font-weight:400;opacity:0}.superset-legacy-chart-heatmap .legendCells .cell:first-child text{opacity:1}.superset-legacy-chart-heatmap .legendCells .cell:last-child text{opacity:1}.dashboard .superset-legacy-chart-heatmap .axis text{font-size:10px;opacity:.75}.superset-legacy-chart-horizon{overflow:auto;position:relative}.superset-legacy-chart-horizon .horizon-row{border-bottom:1px solid #ddd;border-top:0;padding:0;margin:0}.superset-legacy-chart-horizon .horizon-row span.title{position:absolute;color:#333;font-size:.8em;margin:0;text-shadow:1px 1px hsla(0,0%,100%,.75)}.mapboxgl-map{font:12px/20px Helvetica Neue,Arial,Helvetica,sans-serif;overflow:hidden;position:relative;-webkit-tap-highlight-color:rgba(0,0,0,0)}.mapboxgl-map:-webkit-full-screen{width:100%;height:100%}.mapboxgl-canary{background-color:salmon}.mapboxgl-canvas-container.mapboxgl-interactive,.mapboxgl-ctrl-group>button.mapboxgl-ctrl-compass{cursor:-webkit-grab;cursor:-moz-grab;cursor:grab;-moz-user-select:none;-webkit-user-select:none;-ms-user-select:none;user-select:none}.mapboxgl-canvas-container.mapboxgl-interactive:active,.mapboxgl-ctrl-group>button.mapboxgl-ctrl-compass:active{cursor:-webkit-grabbing;cursor:-moz-grabbing;cursor:grabbing}.mapboxgl-canvas-container.mapboxgl-touch-zoom-rotate,.mapboxgl-canvas-container.mapboxgl-touch-zoom-rotate .mapboxgl-canvas{touch-action:pan-x pan-y}.mapboxgl-canvas-container.mapboxgl-touch-drag-pan,.mapboxgl-canvas-container.mapboxgl-touch-drag-pan .mapboxgl-canvas{touch-action:pinch-zoom}.mapboxgl-canvas-container.mapboxgl-touch-zoom-rotate.mapboxgl-touch-drag-pan,.mapboxgl-canvas-container.mapboxgl-touch-zoom-rotate.mapboxgl-touch-drag-pan .mapboxgl-canvas{touch-action:none}.mapboxgl-ctrl-bottom-left,.mapboxgl-ctrl-bottom-right,.mapboxgl-ctrl-top-left,.mapboxgl-ctrl-top-right{position:absolute;pointer-events:none;z-index:2}.mapboxgl-ctrl-top-left{top:0;left:0}.mapboxgl-ctrl-top-right{top:0;right:0}.mapboxgl-ctrl-bottom-left{bottom:0;left:0}.mapboxgl-ctrl-bottom-right{right:0;bottom:0}.mapboxgl-ctrl{clear:both;pointer-events:auto}.mapboxgl-ctrl-top-left .mapboxgl-ctrl{margin:10px 0 0 10px;float:left}.mapboxgl-ctrl-top-right .mapboxgl-ctrl{margin:10px 10px 0 0;float:right}.mapboxgl-ctrl-bottom-left .mapboxgl-ctrl{margin:0 0 10px 10px;float:left}.mapboxgl-ctrl-bottom-right .mapboxgl-ctrl{margin:0 10px 10px 0;float:right}.mapboxgl-ctrl-group{border-radius:4px;overflow:hidden;background:#fff}.mapboxgl-ctrl-group:not(:empty){-moz-box-shadow:0 0 2px rgba(0,0,0,.1);-webkit-box-shadow:0 0 2px rgba(0,0,0,.1);box-shadow:0 0 0 2px rgba(0,0,0,.1)}.mapboxgl-ctrl-group>button{width:30px;height:30px;display:block;padding:0;outline:none;border:0;box-sizing:border-box;background-color:transparent;cursor:pointer}.mapboxgl-ctrl-group>button+button{border-top:1px solid #ddd}.mapboxgl-ctrl>button::-moz-focus-inner{border:0;padding:0}.mapboxgl-ctrl>button:hover{background-color:rgba(0,0,0,.05)}.mapboxgl-ctrl-icon,.mapboxgl-ctrl-icon>.mapboxgl-ctrl-compass-arrow{speak:none;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.mapboxgl-ctrl-icon{padding:5px}.mapboxgl-ctrl-icon.mapboxgl-ctrl-zoom-out{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath d='M7 9c-.554 0-1 .446-1 1s.446 1 1 1h6c.554 0 1-.446 1-1s-.446-1-1-1z' fill='%23333'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-zoom-in{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath d='M10 6c-.554 0-1 .446-1 1v2H7c-.554 0-1 .446-1 1s.446 1 1 1h2v2c0 .554.446 1 1 1s1-.446 1-1v-2h2c.554 0 1-.446 1-1s-.446-1-1-1h-2V7c0-.554-.446-1-1-1z' fill='%23333'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg' fill='%23333'%3E%3Cpath d='M10 4C9 4 9 5 9 5v.1A5 5 0 005.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 009 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 003.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0011 5.1V5s0-1-1-1zm0 2.5a3.5 3.5 0 013.5 3.5 3.5 3.5 0 01-3.5 3.5A3.5 3.5 0 016.5 10 3.5 3.5 0 0110 6.5zm0 1.8A1.8 1.8 0 008.3 10a1.8 1.8 0 001.7 1.8 1.8 1.8 0 001.8-1.8A1.8 1.8 0 0010 8.3z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate:disabled{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg' fill='%23aaa'%3E%3Cpath d='M10 4C9 4 9 5 9 5v.1A5 5 0 005.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 009 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 003.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0011 5.1V5s0-1-1-1zm0 2.5a3.5 3.5 0 013.5 3.5 3.5 3.5 0 01-3.5 3.5A3.5 3.5 0 016.5 10 3.5 3.5 0 0110 6.5zm0 1.8A1.8 1.8 0 008.3 10a1.8 1.8 0 001.7 1.8 1.8 1.8 0 001.8-1.8A1.8 1.8 0 0010 8.3z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate.mapboxgl-ctrl-geolocate-active{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg' fill='%2333b5e5'%3E%3Cpath d='M10 4C9 4 9 5 9 5v.1A5 5 0 005.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 009 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 003.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0011 5.1V5s0-1-1-1zm0 2.5a3.5 3.5 0 013.5 3.5 3.5 3.5 0 01-3.5 3.5A3.5 3.5 0 016.5 10 3.5 3.5 0 0110 6.5zm0 1.8A1.8 1.8 0 008.3 10a1.8 1.8 0 001.7 1.8 1.8 1.8 0 001.8-1.8A1.8 1.8 0 0010 8.3z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate.mapboxgl-ctrl-geolocate-active-error{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg' fill='%23e58978'%3E%3Cpath d='M10 4C9 4 9 5 9 5v.1A5 5 0 005.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 009 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 003.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0011 5.1V5s0-1-1-1zm0 2.5a3.5 3.5 0 013.5 3.5 3.5 3.5 0 01-3.5 3.5A3.5 3.5 0 016.5 10 3.5 3.5 0 0110 6.5zm0 1.8A1.8 1.8 0 008.3 10a1.8 1.8 0 001.7 1.8 1.8 1.8 0 001.8-1.8A1.8 1.8 0 0010 8.3z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate.mapboxgl-ctrl-geolocate-background{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg' fill='%2333b5e5'%3E%3Cpath d='M10 4C9 4 9 5 9 5v.1A5 5 0 005.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 009 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 003.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0011 5.1V5s0-1-1-1zm0 2.5a3.5 3.5 0 110 7 3.5 3.5 0 010-7z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate.mapboxgl-ctrl-geolocate-background-error{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg' fill='%23e54e33'%3E%3Cpath d='M10 4C9 4 9 5 9 5v.1A5 5 0 005.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 009 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 003.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0011 5.1V5s0-1-1-1zm0 2.5a3.5 3.5 0 110 7 3.5 3.5 0 010-7z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-geolocate.mapboxgl-ctrl-geolocate-waiting{-webkit-animation:mapboxgl-spin 2s linear infinite;-moz-animation:mapboxgl-spin 2s infinite linear;-o-animation:mapboxgl-spin 2s infinite linear;-ms-animation:mapboxgl-spin 2s infinite linear;animation:mapboxgl-spin 2s linear infinite}@-webkit-keyframes mapboxgl-spin{0%{-webkit-transform:rotate(0deg)}to{-webkit-transform:rotate(1turn)}}@-moz-keyframes mapboxgl-spin{0%{-moz-transform:rotate(0deg)}to{-moz-transform:rotate(1turn)}}@-o-keyframes mapboxgl-spin{0%{-o-transform:rotate(0deg)}to{-o-transform:rotate(1turn)}}@-ms-keyframes mapboxgl-spin{0%{-ms-transform:rotate(0deg)}to{-ms-transform:rotate(1turn)}}@keyframes mapboxgl-spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}.mapboxgl-ctrl-icon.mapboxgl-ctrl-fullscreen{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath d='M5 4c-.5 0-1 .5-1 1v4h.5l1.277-1.703c1 .763 2.059 1.66 3.114 2.703a30.38 30.38 0 01-3.11 2.707L4.5 11H4v4c0 .5.5 1 1 1h4v-.5l-1.727-1.295a35.498 35.498 0 012.688-3.137 30.312 30.312 0 012.746 3.15L11 15.5v.5h4c.5 0 1-.5 1-1v-4h-.5l-1.295 1.727a35.513 35.513 0 01-3.168-2.717 47.787 47.787 0 013.192-2.705L15.5 9h.5V5c0-.5-.5-1-1-1h-4v.5l1.703 1.277A32.243 32.243 0 019.971 8.92 47.761 47.761 0 017.305 5.77L9 4.5V4H5z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-shrink{background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath d='M4.242 3.492a.75.75 0 00-.523 1.29l2.246 2.245L4 8.5V9h4c.5 0 1-.5 1-1V4h-.5L7.018 5.955 4.78 3.72a.75.75 0 00-.539-.227zm11.492 0a.75.75 0 00-.515.227l-2.235 2.234L11.5 4H11v4c0 .5.5 1 1 1h4v-.5l-1.965-1.473 2.246-2.246a.75.75 0 00-.547-1.289zM4 11v.5l1.965 1.473-2.246 2.246A.751.751 0 104.78 16.28l2.246-2.246L8.5 16H9v-4c0-.5-.5-1-1-1H4zm8 0c-.5 0-1 .5-1 1v4h.5l1.473-1.965 2.246 2.246a.751.751 0 101.062-1.062l-2.246-2.246L16 11.5V11h-4z'/%3E%3C/svg%3E")}.mapboxgl-ctrl-icon.mapboxgl-ctrl-compass>.mapboxgl-ctrl-compass-arrow{width:20px;height:20px;margin:5px;background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath fill='%23333' d='M6 9l4-8 4 8z'/%3E%3Cpath fill='%23CCC' d='M6 11l4 8 4-8z'/%3E%3C/svg%3E");background-repeat:no-repeat;display:inline-block}a.mapboxgl-ctrl-logo{width:85px;height:21px;margin:0 0 -3px -3px;display:block;background-repeat:no-repeat;cursor:pointer;background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 84.49 21'%3E%3Cpath class='st0' d='M83.25 14.26c0 .12-.09.21-.21.21h-1.61c-.13 0-.24-.06-.3-.17l-1.44-2.39-1.44 2.39a.34.34 0 01-.3.17h-1.61c-.04 0-.08-.01-.12-.03-.09-.06-.13-.19-.06-.28l2.43-3.68-2.39-3.64a.213.213 0 01-.03-.12c0-.12.09-.21.21-.21h1.61c.13 0 .24.06.3.17l1.41 2.36 1.4-2.35a.34.34 0 01.3-.17H83c.04 0 .08.01.12.03.09.06.13.19.06.28l-2.37 3.63 2.43 3.67c0 .05.01.09.01.13zM66.24 9.59c-.39-1.88-1.96-3.28-3.84-3.28-1.03 0-2.03.42-2.73 1.18V3.51c0-.13-.1-.23-.23-.23h-1.4c-.13 0-.23.11-.23.23v10.72c0 .13.1.23.23.23h1.4c.13 0 .23-.11.23-.23v-.73c.71.75 1.7 1.18 2.73 1.18 1.88 0 3.45-1.41 3.84-3.29.13-.6.13-1.21 0-1.8zM62.08 13c-1.32 0-2.39-1.11-2.41-2.48v-.06c.02-1.38 1.09-2.48 2.41-2.48s2.42 1.12 2.42 2.51S63.41 13 62.08 13zM71.67 6.32a4.24 4.24 0 00-4.16 3.29c-.13.59-.13 1.19 0 1.77a4.233 4.233 0 004.17 3.3c2.35 0 4.26-1.87 4.26-4.19s-1.9-4.17-4.27-4.17zm-.02 6.69c-1.33 0-2.42-1.12-2.42-2.51s1.08-2.52 2.42-2.52c1.33 0 2.42 1.12 2.42 2.51s-1.08 2.51-2.42 2.52z' opacity='.9' fill='%23fff'/%3E%3Cpath class='st1' d='M62.08 7.98c-1.32 0-2.39 1.11-2.41 2.48v.06c.01 1.38 1.08 2.48 2.41 2.48s2.42-1.12 2.42-2.51-1.09-2.51-2.42-2.51zm0 3.78c-.63 0-1.14-.56-1.17-1.25v-.04c.01-.69.54-1.25 1.17-1.25.63 0 1.17.57 1.17 1.27-.01.71-.52 1.27-1.17 1.27zM71.65 7.98c-1.33 0-2.42 1.12-2.42 2.51S70.32 13 71.65 13s2.42-1.12 2.42-2.51-1.08-2.51-2.42-2.51zm0 3.78c-.64 0-1.17-.57-1.17-1.27 0-.7.53-1.26 1.17-1.26s1.17.57 1.17 1.27c0 .71-.53 1.26-1.17 1.26z' opacity='.35'/%3E%3Cpath class='st0' d='M45.74 6.53h-1.4c-.13 0-.23.11-.23.23v.73c-.71-.75-1.7-1.18-2.73-1.18-2.17 0-3.94 1.87-3.94 4.19s1.77 4.19 3.94 4.19c1.04 0 2.03-.43 2.73-1.19v.73c0 .13.1.23.23.23h1.4c.13 0 .23-.11.23-.23V6.74c0-.12-.09-.22-.22-.22 0 .01 0 .01-.01.01zm-1.62 4C44.11 11.9 43.03 13 41.71 13s-2.42-1.12-2.42-2.51 1.08-2.52 2.4-2.52c1.33 0 2.39 1.11 2.41 2.48l.02.08z' opacity='.9' fill='%23fff'/%3E%3Cpath class='st1' d='M41.71 7.98c-1.33 0-2.42 1.12-2.42 2.51S40.37 13 41.71 13s2.39-1.11 2.41-2.48v-.06c-.02-1.37-1.09-2.48-2.41-2.48zm-1.16 2.51c0-.7.52-1.27 1.17-1.27.64 0 1.14.56 1.17 1.25v.04c-.01.68-.53 1.24-1.17 1.24-.64 0-1.17-.56-1.17-1.26z' opacity='.35'/%3E%3Cpath class='st0' d='M52.41 6.32c-1.03 0-2.03.42-2.73 1.18v-.75c0-.13-.1-.23-.23-.23h-1.4c-.13 0-.23.11-.23.23v10.72c0 .13.1.23.23.23h1.4c.13 0 .23-.1.23-.23V13.5c.71.75 1.7 1.18 2.74 1.18 2.17 0 3.94-1.87 3.94-4.19s-1.78-4.17-3.95-4.17zm-.33 6.69c-1.32 0-2.39-1.11-2.42-2.48v-.07c.02-1.38 1.09-2.49 2.4-2.49 1.32 0 2.41 1.12 2.41 2.51S53.4 13 52.08 13.01z' opacity='.9' fill='%23fff'/%3E%3Cpath class='st1' d='M52.08 7.98c-1.32 0-2.39 1.11-2.42 2.48v.06c.03 1.38 1.1 2.48 2.42 2.48s2.41-1.12 2.41-2.51-1.09-2.51-2.41-2.51zm0 3.78c-.63 0-1.14-.56-1.17-1.25v-.04c.01-.69.54-1.25 1.17-1.25.63 0 1.17.58 1.17 1.27s-.53 1.27-1.17 1.27z' opacity='.35'/%3E%3Cpath class='st0' d='M36.08 14.24c0 .13-.1.23-.23.23h-1.41c-.13 0-.23-.11-.23-.23V9.68c0-.98-.74-1.71-1.62-1.71-.8 0-1.46.7-1.59 1.62l.01 4.66c0 .13-.11.23-.23.23h-1.41c-.13 0-.23-.11-.23-.23V9.68c0-.98-.74-1.71-1.62-1.71-.85 0-1.54.79-1.6 1.8v4.48c0 .13-.1.23-.23.23h-1.4c-.13 0-.23-.11-.23-.23V6.74c.01-.13.1-.22.23-.22h1.4c.13 0 .22.11.23.22v.66c.5-.68 1.3-1.09 2.16-1.1h.03c1.09 0 2.09.6 2.6 1.55.45-.95 1.4-1.55 2.44-1.56 1.62 0 2.93 1.25 2.9 2.78l.03 5.17z' opacity='.9' fill='%23fff'/%3E%3Cpath class='st1' d='M84.34 13.59l-.07-.13-1.96-2.99 1.94-2.95c.44-.67.26-1.56-.41-2.02-.02 0-.03 0-.04-.01-.23-.15-.5-.22-.78-.22h-1.61c-.56 0-1.08.29-1.37.78l-.32.55-.34-.56c-.29-.48-.81-.77-1.38-.77h-1.6c-.6 0-1.13.37-1.35.92a5.59 5.59 0 00-7.26.45c-.35.34-.65.72-.89 1.14-.9-1.62-2.58-2.72-4.5-2.72-.5 0-1.01.07-1.48.23V3.51c0-.82-.66-1.48-1.47-1.48h-1.4c-.81 0-1.47.66-1.47 1.47v3.75a5.12 5.12 0 00-4.17-2.19c-.74 0-1.46.16-2.12.47-.24-.17-.54-.26-.84-.26h-1.4c-.45 0-.87.21-1.15.56a1.498 1.498 0 00-1.16-.55h-1.39c-.3 0-.6.09-.84.26-.67-.3-1.39-.46-2.12-.46-1.83 0-3.43 1-4.37 2.5-.2-.46-.48-.89-.83-1.25-.8-.81-1.89-1.25-3.02-1.25h-.01c-.89.01-1.75.33-2.46.88-.74-.57-1.64-.88-2.57-.88h-.03c-.29 0-.58.03-.86.11-.28.06-.56.16-.82.28-.21-.12-.45-.18-.7-.18h-1.4c-.82 0-1.47.66-1.47 1.47v7.5c0 .82.66 1.47 1.47 1.47h1.4c.82 0 1.48-.66 1.48-1.48V9.79c.03-.36.23-.59.36-.59.18 0 .38.18.38.47v4.57c0 .82.66 1.47 1.47 1.47h1.41c.82 0 1.47-.66 1.47-1.47l-.01-4.57c.06-.32.25-.47.35-.47.18 0 .38.18.38.47v4.57c0 .82.66 1.47 1.47 1.47h1.41c.82 0 1.47-.66 1.47-1.47v-.38a5.068 5.068 0 004.06 2.06c.74 0 1.46-.16 2.12-.47.24.17.54.26.84.26h1.39c.3 0 .6-.09.84-.26v2.01c0 .82.66 1.47 1.47 1.47h1.4c.82 0 1.47-.66 1.47-1.47v-1.77c.48.15.99.23 1.49.22 1.7 0 3.22-.87 4.17-2.2v.52c0 .82.66 1.47 1.47 1.47h1.4c.3 0 .6-.09.84-.26.66.31 1.39.47 2.12.47 1.92 0 3.6-1.1 4.49-2.73 1.54 2.65 4.95 3.53 7.58 1.98.18-.11.36-.22.53-.36.22.55.76.91 1.35.9H78c.56 0 1.08-.29 1.37-.78l.37-.61.37.61c.29.48.81.78 1.38.78h1.6c.81 0 1.46-.66 1.45-1.46-.05-.22-.1-.44-.2-.65zm-48.48.88h-1.41c-.13 0-.23-.11-.23-.23V9.68c0-.98-.74-1.71-1.62-1.71-.8 0-1.46.7-1.59 1.62l.01 4.66c0 .13-.1.23-.23.23h-1.41c-.13 0-.23-.11-.23-.23V9.68c0-.98-.74-1.71-1.62-1.71-.85 0-1.54.79-1.6 1.8v4.48c0 .13-.1.23-.23.23h-1.4c-.13 0-.23-.11-.23-.23V6.74a.23.23 0 01.23-.22h1.4c.13 0 .22.11.23.22v.66c.5-.68 1.3-1.09 2.16-1.1h.03c1.09 0 2.09.6 2.6 1.55.45-.95 1.4-1.55 2.44-1.56 1.62 0 2.93 1.25 2.9 2.78l.01 5.16c.02.13-.09.23-.21.24zm10.11-.23c0 .13-.1.23-.23.23h-1.4c-.13 0-.23-.11-.23-.23v-.74c-.7.76-1.69 1.18-2.72 1.18-2.17 0-3.94-1.87-3.94-4.19s1.77-4.19 3.94-4.19c1.03 0 2.02.43 2.73 1.18v-.74c0-.13.1-.23.23-.23h1.4c.12-.01.22.08.23.21V14.25h-.01v-.01zm6.44.43c-1.03 0-2.02-.43-2.73-1.18v3.97c0 .13-.1.23-.23.23h-1.4c-.13 0-.23-.1-.23-.23V6.75c0-.13.1-.22.23-.22h1.4c.13 0 .23.11.23.23v.73a3.73 3.73 0 012.73-1.18c2.17 0 3.94 1.86 3.94 4.18s-1.77 4.18-3.94 4.18zm13.83-3.28c-.39 1.87-1.96 3.29-3.84 3.29-1.03 0-2.02-.43-2.73-1.18v.73c0 .13-.1.23-.23.23h-1.4c-.13 0-.23-.11-.23-.23V3.51c0-.13.1-.23.23-.23h1.4c.13 0 .23.11.23.23v3.97a3.72 3.72 0 012.73-1.17c1.88 0 3.45 1.4 3.84 3.28.13.6.13 1.21 0 1.8zm5.43 3.29c-2 .01-3.73-1.35-4.17-3.3-.13-.59-.13-1.19 0-1.77a4.243 4.243 0 014.17-3.3c2.36 0 4.26 1.87 4.26 4.19s-1.9 4.18-4.26 4.18zm11.37-.21h-1.61c-.13 0-.24-.06-.3-.17l-1.44-2.39-1.44 2.39a.34.34 0 01-.3.17h-1.61c-.04 0-.08-.01-.12-.03-.09-.06-.13-.19-.06-.28l2.43-3.68-2.39-3.64a.213.213 0 01-.03-.12c0-.12.09-.21.21-.21h1.61c.13 0 .24.06.3.17l1.41 2.36 1.41-2.36a.34.34 0 01.3-.17h1.61c.04 0 .08.01.12.03.09.06.13.19.06.28l-2.38 3.64 2.43 3.67c.02.03.03.07.03.12-.03.13-.12.22-.24.22z' opacity='.35'/%3E%3Cpath class='st0' d='M10.5 1.24c-5.11 0-9.25 4.15-9.25 9.25s4.15 9.25 9.25 9.25 9.25-4.15 9.25-9.25c0-5.11-4.14-9.25-9.25-9.25zm4.39 11.53c-1.93 1.93-4.78 2.31-6.7 2.31-.7 0-1.41-.05-2.1-.16 0 0-1.02-5.64 2.14-8.81a4.4 4.4 0 013.13-1.28c1.27 0 2.49.51 3.39 1.42 1.84 1.84 1.89 4.75.14 6.52z' opacity='.9' fill='%23fff'/%3E%3Cpath class='st1' d='M10.5-.01C4.7-.01 0 4.7 0 10.49s4.7 10.5 10.5 10.5S21 16.29 21 10.49C20.99 4.7 16.3-.01 10.5-.01zm0 19.75c-5.11 0-9.25-4.15-9.25-9.25s4.14-9.26 9.25-9.26 9.25 4.15 9.25 9.25c0 5.13-4.14 9.26-9.25 9.26z' opacity='.35'/%3E%3Cpath class='st1' d='M14.74 6.25c-1.84-1.84-4.76-1.9-6.51-.15-3.16 3.17-2.14 8.81-2.14 8.81s5.64 1.02 8.81-2.14c1.74-1.77 1.69-4.68-.16-6.52zm-2.27 4.09l-.91 1.87-.9-1.87-1.86-.91 1.86-.9.9-1.87.91 1.87 1.86.9-1.86.91z' opacity='.35'/%3E%3Cpath class='st0' opacity='.9' fill='%23fff' d='M14.33 9.43l-1.86.91-.91 1.87-.9-1.87-1.86-.91 1.86-.9.9-1.87.91 1.87z'/%3E%3C/svg%3E")}a.mapboxgl-ctrl-logo.mapboxgl-compact{width:21px;height:21px;background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 21 21'%3E%3Cpath d='M10.5 1.25c-5.11 0-9.25 4.15-9.25 9.25s4.15 9.25 9.25 9.25 9.25-4.15 9.25-9.25c0-5.11-4.14-9.25-9.25-9.25zm4.39 11.53c-1.93 1.93-4.78 2.31-6.7 2.31-.7 0-1.41-.05-2.1-.16 0 0-1.02-5.64 2.14-8.81a4.4 4.4 0 013.13-1.28c1.27 0 2.49.51 3.39 1.42 1.84 1.84 1.89 4.75.14 6.52z' class='st0' opacity='.9' fill='%23fff'/%3E%3Cpath d='M10.5 0C4.7 0 0 4.71 0 10.5S4.7 21 10.5 21 21 16.3 21 10.5C20.99 4.71 16.3 0 10.5 0zm0 19.75c-5.11 0-9.25-4.15-9.25-9.25s4.14-9.26 9.25-9.26 9.25 4.15 9.25 9.25c0 5.13-4.14 9.26-9.25 9.26z' class='st1' opacity='.35'/%3E%3Cpath d='M14.74 6.26c-1.84-1.84-4.76-1.9-6.51-.15-3.16 3.17-2.14 8.81-2.14 8.81s5.64 1.02 8.81-2.14c1.74-1.77 1.69-4.68-.16-6.52zm-2.27 4.09l-.91 1.87-.9-1.87-1.86-.91 1.86-.9.9-1.87.91 1.87 1.86.9z' class='st1' opacity='.35'/%3E%3Cpath class='st0' opacity='.9' fill='%23fff' d='M11.56 12.22l-.9-1.87-1.86-.91 1.86-.9.9-1.87.91 1.87 1.86.9-1.86.91z'/%3E%3C/svg%3E")}.mapboxgl-ctrl.mapboxgl-ctrl-attrib{padding:0 5px;background-color:hsla(0,0%,100%,.5);margin:0}@media screen{.mapboxgl-ctrl-attrib.mapboxgl-compact{margin:10px;position:relative;background-color:#fff;border-radius:3px 12px 12px 3px}.mapboxgl-ctrl-attrib.mapboxgl-compact:hover{padding:2px 24px 2px 4px;visibility:visible}.mapboxgl-ctrl-bottom-left>.mapboxgl-ctrl-attrib.mapboxgl-compact:hover,.mapboxgl-ctrl-top-left>.mapboxgl-ctrl-attrib.mapboxgl-compact:hover{padding:2px 4px 2px 24px;border-radius:12px 3px 3px 12px}.mapboxgl-ctrl-attrib.mapboxgl-compact .mapboxgl-ctrl-attrib-inner{display:none}.mapboxgl-ctrl-attrib.mapboxgl-compact:hover .mapboxgl-ctrl-attrib-inner{display:block}.mapboxgl-ctrl-attrib.mapboxgl-compact:after{content:"";cursor:pointer;position:absolute;background-image:url("data:image/svg+xml;charset=utf-8,%3Csvg viewBox='0 0 20 20' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath fill='%23333' fill-rule='evenodd' d='M4 10a6 6 0 1012 0 6 6 0 10-12 0m5-3a1 1 0 102 0 1 1 0 10-2 0m0 3a1 1 0 112 0v3a1 1 0 11-2 0'/%3E%3C/svg%3E");background-color:hsla(0,0%,100%,.5);width:24px;height:24px;box-sizing:border-box;border-radius:12px}.mapboxgl-ctrl-bottom-right>.mapboxgl-ctrl-attrib.mapboxgl-compact:after{bottom:0;right:0}.mapboxgl-ctrl-top-right>.mapboxgl-ctrl-attrib.mapboxgl-compact:after{top:0;right:0}.mapboxgl-ctrl-top-left>.mapboxgl-ctrl-attrib.mapboxgl-compact:after{top:0;left:0}.mapboxgl-ctrl-bottom-left>.mapboxgl-ctrl-attrib.mapboxgl-compact:after{bottom:0;left:0}}.mapboxgl-ctrl-attrib a{color:rgba(0,0,0,.75);text-decoration:none}.mapboxgl-ctrl-attrib a:hover{color:inherit;text-decoration:underline}.mapboxgl-ctrl-attrib .mapbox-improve-map{font-weight:700;margin-left:2px}.mapboxgl-attrib-empty{display:none}.mapboxgl-ctrl-scale{background-color:hsla(0,0%,100%,.75);font-size:10px;border:2px solid #333;border-top:#333;padding:0 5px;color:#333;box-sizing:border-box}.mapboxgl-popup{position:absolute;top:0;left:0;display:-webkit-flex;display:flex;will-change:transform;pointer-events:none}.mapboxgl-popup-anchor-top,.mapboxgl-popup-anchor-top-left,.mapboxgl-popup-anchor-top-right{-webkit-flex-direction:column;flex-direction:column}.mapboxgl-popup-anchor-bottom,.mapboxgl-popup-anchor-bottom-left,.mapboxgl-popup-anchor-bottom-right{-webkit-flex-direction:column-reverse;flex-direction:column-reverse}.mapboxgl-popup-anchor-left{-webkit-flex-direction:row;flex-direction:row}.mapboxgl-popup-anchor-right{-webkit-flex-direction:row-reverse;flex-direction:row-reverse}.mapboxgl-popup-tip{width:0;height:0;border:10px solid transparent;z-index:1}.mapboxgl-popup-anchor-top .mapboxgl-popup-tip{-webkit-align-self:center;align-self:center;border-top:none;border-bottom-color:#fff}.mapboxgl-popup-anchor-top-left .mapboxgl-popup-tip{-webkit-align-self:flex-start;align-self:flex-start;border-top:none;border-left:none;border-bottom-color:#fff}.mapboxgl-popup-anchor-top-right .mapboxgl-popup-tip{-webkit-align-self:flex-end;align-self:flex-end;border-top:none;border-right:none;border-bottom-color:#fff}.mapboxgl-popup-anchor-bottom .mapboxgl-popup-tip{-webkit-align-self:center;align-self:center;border-bottom:none;border-top-color:#fff}.mapboxgl-popup-anchor-bottom-left .mapboxgl-popup-tip{-webkit-align-self:flex-start;align-self:flex-start;border-bottom:none;border-left:none;border-top-color:#fff}.mapboxgl-popup-anchor-bottom-right .mapboxgl-popup-tip{-webkit-align-self:flex-end;align-self:flex-end;border-bottom:none;border-right:none;border-top-color:#fff}.mapboxgl-popup-anchor-left .mapboxgl-popup-tip{-webkit-align-self:center;align-self:center;border-left:none;border-right-color:#fff}.mapboxgl-popup-anchor-right .mapboxgl-popup-tip{-webkit-align-self:center;align-self:center;border-right:none;border-left-color:#fff}.mapboxgl-popup-close-button{position:absolute;right:0;top:0;border:0;border-radius:0 3px 0 0;cursor:pointer;background-color:transparent}.mapboxgl-popup-close-button:hover{background-color:rgba(0,0,0,.05)}.mapboxgl-popup-content{position:relative;background:#fff;border-radius:3px;box-shadow:0 1px 2px rgba(0,0,0,.1);padding:10px 10px 15px;pointer-events:auto}.mapboxgl-popup-anchor-top-left .mapboxgl-popup-content{border-top-left-radius:0}.mapboxgl-popup-anchor-top-right .mapboxgl-popup-content{border-top-right-radius:0}.mapboxgl-popup-anchor-bottom-left .mapboxgl-popup-content{border-bottom-left-radius:0}.mapboxgl-popup-anchor-bottom-right .mapboxgl-popup-content{border-bottom-right-radius:0}.mapboxgl-marker{position:absolute;top:0;left:0;will-change:transform}.mapboxgl-user-location-dot{background-color:#1da1f2;width:15px;height:15px;border-radius:50%;box-shadow:0 0 2px rgba(0,0,0,.25)}.mapboxgl-user-location-dot:before{background-color:#1da1f2;content:"";width:15px;height:15px;border-radius:50%;position:absolute;-webkit-animation:mapboxgl-user-location-dot-pulse 2s infinite;-moz-animation:mapboxgl-user-location-dot-pulse 2s infinite;-ms-animation:mapboxgl-user-location-dot-pulse 2s infinite;animation:mapboxgl-user-location-dot-pulse 2s infinite}.mapboxgl-user-location-dot:after{border-radius:50%;border:2px solid #fff;content:"";height:19px;left:-2px;position:absolute;top:-2px;width:19px;box-sizing:border-box}@-webkit-keyframes mapboxgl-user-location-dot-pulse{0%{-webkit-transform:scale(1);opacity:1}70%{-webkit-transform:scale(3);opacity:0}to{-webkit-transform:scale(1);opacity:0}}@-ms-keyframes mapboxgl-user-location-dot-pulse{0%{-ms-transform:scale(1);opacity:1}70%{-ms-transform:scale(3);opacity:0}to{-ms-transform:scale(1);opacity:0}}@keyframes mapboxgl-user-location-dot-pulse{0%{transform:scale(1);opacity:1}70%{transform:scale(3);opacity:0}to{transform:scale(1);opacity:0}}.mapboxgl-user-location-dot-stale{background-color:#aaa}.mapboxgl-user-location-dot-stale:after{display:none}.mapboxgl-crosshair,.mapboxgl-crosshair .mapboxgl-interactive,.mapboxgl-crosshair .mapboxgl-interactive:active{cursor:crosshair}.mapboxgl-boxzoom{position:absolute;top:0;left:0;width:0;height:0;background:#fff;border:2px dotted #202020;opacity:.5}@media print{.mapbox-improve-map{display:none}}.mapbox .slice_container div{padding-top:0}.superset-legacy-chart-paired_ttest .scrollbar-container{overflow:auto}.paired-ttest-table .scrollbar-content{padding-left:5px;padding-right:5px;margin-bottom:0}.paired-ttest-table table{margin-bottom:0}.paired-ttest-table h1{margin-left:5px}.reactable-data tr,.reactable-header-sortable{-webkit-transition:.1s ease-in-out;transition:.1s ease-in-out}.reactable-data tr:hover{background-color:#e0e0e0}.reactable-data tr .false{color:#f44336}.reactable-data tr .true{color:#4caf50}.reactable-data tr .control{color:#2196f3}.reactable-data tr .invalid{color:#ff9800}.reactable-data .control td{background-color:#eee}.reactable-header-sort-asc,.reactable-header-sort-desc,.reactable-header-sortable:focus,.reactable-header-sortable:hover{background-color:#e0e0e0;position:relative}.reactable-header-sort-asc:after{content:"\25BC";position:absolute;right:10px}.reactable-header-sort-desc:after{content:"\25B2";position:absolute;right:10px}.parcoords canvas,.parcoords svg{font-size:12px;position:absolute}.parcoords>canvas{pointer-events:none}.parcoords text.label{font:100%;font-size:12px;cursor:drag}.parcoords rect.background{fill:transparent}.parcoords rect.background:hover{fill:hsla(0,0%,47.1%,.2)}.parcoords .resize rect{fill:rgba(0,0,0,.1)}.parcoords rect.extent{fill:hsla(0,0%,100%,.25);stroke:rgba(0,0,0,.6)}.parcoords .axis line,.parcoords .axis path{fill:none;stroke:#222;shape-rendering:crispEdges}.parcoords canvas{opacity:1;-moz-transition:opacity .3s;-webkit-transition:opacity .3s;-o-transition:opacity .3s}.parcoords canvas.faded{opacity:.25}.parcoords{-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;background-color:#fff}.parcoords .header,.parcoords .row{clear:left;font-size:12px;line-height:18px;height:18px;margin:0}.parcoords .row:nth-child(odd){background:rgba(0,0,0,.05)}.parcoords .header{font-weight:700}.parcoords .cell{float:left;overflow:hidden;white-space:nowrap;width:100px;height:18px}.parcoords .col-0{width:180px}.superset-legacy-chart-parallel-coordinates div.grid{overflow:auto}.superset-legacy-chart-parallel-coordinates .grid div.row:hover{background-color:#ccc}.superset-legacy-chart-partition{position:relative}.superset-legacy-chart-partition .chart{display:block;margin:auto;font-size:11px}.superset-legacy-chart-partition rect{stroke:#eee;fill:#aaa;fill-opacity:.8;transition:fill-opacity .18s linear;cursor:pointer}.superset-legacy-chart-partition rect:hover{fill-opacity:1}.superset-legacy-chart-partition g text{font-weight:700;fill:rgba(0,0,0,.8)}.superset-legacy-chart-partition g:hover text{fill:#000}.superset-legacy-chart-partition .partition-tooltip{position:absolute;top:0;left:0;opacity:0;padding:5px;pointer-events:none;background-color:hsla(0,0%,100%,.75);border-radius:5px}.partition-tooltip td{padding-left:5px;font-size:11px}table.dataTable{clear:both;margin-top:6px!important;margin-bottom:6px!important;max-width:none!important;border-collapse:separate!important}table.dataTable td,table.dataTable th{-webkit-box-sizing:content-box;box-sizing:content-box}table.dataTable td.dataTables_empty,table.dataTable th.dataTables_empty{text-align:center}table.dataTable.nowrap td,table.dataTable.nowrap th{white-space:nowrap}div.dataTables_wrapper div.dataTables_length label{font-weight:400;text-align:left;white-space:nowrap}div.dataTables_wrapper div.dataTables_length select{width:75px;display:inline-block}div.dataTables_wrapper div.dataTables_filter{text-align:right}div.dataTables_wrapper div.dataTables_filter label{font-weight:400;white-space:nowrap;text-align:left}div.dataTables_wrapper div.dataTables_filter input{margin-left:.5em;display:inline-block;width:auto}div.dataTables_wrapper div.dataTables_info{padding-top:8px;white-space:nowrap}div.dataTables_wrapper div.dataTables_paginate{margin:0;white-space:nowrap;text-align:right}div.dataTables_wrapper div.dataTables_paginate ul.pagination{margin:2px 0;white-space:nowrap}div.dataTables_wrapper div.dataTables_processing{position:absolute;top:50%;left:50%;width:200px;margin-left:-100px;margin-top:-26px;text-align:center;padding:1em 0}table.dataTable thead>tr>td.sorting,table.dataTable thead>tr>td.sorting_asc,table.dataTable thead>tr>td.sorting_desc,table.dataTable thead>tr>th.sorting,table.dataTable thead>tr>th.sorting_asc,table.dataTable thead>tr>th.sorting_desc{padding-right:30px}table.dataTable thead>tr>td:active,table.dataTable thead>tr>th:active{outline:none}table.dataTable thead .sorting,table.dataTable thead .sorting_asc,table.dataTable thead .sorting_asc_disabled,table.dataTable thead .sorting_desc,table.dataTable thead .sorting_desc_disabled{cursor:pointer;position:relative}table.dataTable thead .sorting:after,table.dataTable thead .sorting_asc:after,table.dataTable thead .sorting_asc_disabled:after,table.dataTable thead .sorting_desc:after,table.dataTable thead .sorting_desc_disabled:after{position:absolute;bottom:8px;right:8px;display:block;font-family:Glyphicons Halflings;opacity:.5}table.dataTable thead .sorting:after{opacity:.2;content:"\E150"}table.dataTable thead .sorting_asc:after{content:"\E155"}table.dataTable thead .sorting_desc:after{content:"\E156"}table.dataTable thead .sorting_asc_disabled:after,table.dataTable thead .sorting_desc_disabled:after{color:#eee}div.dataTables_scrollHead table.dataTable{margin-bottom:0!important}div.dataTables_scrollBody>table{border-top:none;margin-top:0!important;margin-bottom:0!important}div.dataTables_scrollBody>table>thead .sorting:after,div.dataTables_scrollBody>table>thead .sorting_asc:after,div.dataTables_scrollBody>table>thead .sorting_desc:after{display:none}div.dataTables_scrollBody>table>tbody>tr:first-child>td,div.dataTables_scrollBody>table>tbody>tr:first-child>th{border-top:none}div.dataTables_scrollFoot>.dataTables_scrollFootInner{box-sizing:content-box}div.dataTables_scrollFoot>.dataTables_scrollFootInner>table{margin-top:0!important;border-top:none}@media screen and (max-width:767px){div.dataTables_wrapper div.dataTables_filter,div.dataTables_wrapper div.dataTables_info,div.dataTables_wrapper div.dataTables_length,div.dataTables_wrapper div.dataTables_paginate{text-align:center}}table.dataTable.table-condensed>thead>tr>th{padding-right:20px}table.dataTable.table-condensed .sorting:after,table.dataTable.table-condensed .sorting_asc:after,table.dataTable.table-condensed .sorting_desc:after{top:6px;right:6px}table.table-bordered.dataTable{border-right-width:0}table.table-bordered.dataTable td,table.table-bordered.dataTable th{border-left-width:0}table.table-bordered.dataTable td:last-child,table.table-bordered.dataTable th:last-child{border-right-width:1px}div.dataTables_scrollHead table.table-bordered,table.table-bordered.dataTable tbody td,table.table-bordered.dataTable tbody th{border-bottom-width:0}div.table-responsive>div.dataTables_wrapper>div.row{margin:0}div.table-responsive>div.dataTables_wrapper>div.row>div[class^=col-]:first-child{padding-left:0}div.table-responsive>div.dataTables_wrapper>div.row>div[class^=col-]:last-child{padding-right:0}.superset-legacy-chart-rose path{transition:fill-opacity .18s linear;stroke:#fff;stroke-width:1px;stroke-opacity:1;fill-opacity:.75}.superset-legacy-chart-rose text{font:400 12px Arial,sans-serif;pointer-events:none}.superset-legacy-chart-rose .clickable path{cursor:pointer}.superset-legacy-chart-rose .hover path{fill-opacity:1}.nv-legend .nv-series{cursor:pointer}.superset-legacy-chart-sankey .node rect{cursor:move;fill-opacity:.9;shape-rendering:crispEdges}.superset-legacy-chart-sankey .node text{pointer-events:none;text-shadow:0 1px 0 #fff}.superset-legacy-chart-sankey .link{fill:none;stroke:#000;stroke-opacity:.2}.superset-legacy-chart-sankey .link:hover{stroke-opacity:.5}.superset-legacy-chart-sankey-tooltip{position:absolute;width:auto;background:#ddd;padding:10px;font-size:12px;font-weight:200;color:#333;border:1px solid #fff;text-align:center;pointer-events:none}.superset-legacy-chart-sunburst text{text-rendering:optimizeLegibility}.superset-legacy-chart-sunburst path{stroke:#ddd;stroke-width:.5px}.superset-legacy-chart-sunburst .center-label{text-anchor:middle;fill:#333;pointer-events:none}.superset-legacy-chart-sunburst .path-abs-percent{font-size:3em;font-weight:700}.superset-legacy-chart-sunburst .path-cond-percent{font-size:2em}.superset-legacy-chart-sunburst .path-metrics,.superset-legacy-chart-sunburst .path-ratio{color:#777}.superset-legacy-chart-sunburst .breadcrumbs text{font-weight:600;font-size:1.2em;text-anchor:middle;fill:#333}.superset-legacy-chart-treemap text{font-size:11px;pointer-events:none}.superset-legacy-chart-treemap tspan:last-child{font-size:9px;fill-opacity:.8}.superset-legacy-chart-treemap .node rect{shape-rendering:crispEdges}.superset-legacy-chart-treemap .node--hover rect{stroke:#000}.react-checkbox-tree{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse;font-size:16px}.react-checkbox-tree>ol{-webkit-box-flex:1;-ms-flex:1 1 auto;flex:1 1 auto}.react-checkbox-tree ol{margin:0;padding-left:0;list-style-type:none}.react-checkbox-tree ol ol{padding-left:24px}.react-checkbox-tree button{line-height:normal;color:inherit}.react-checkbox-tree button:focus{outline:none}.react-checkbox-tree button:disabled{cursor:not-allowed}.react-checkbox-tree .rct-bare-label{cursor:default}.react-checkbox-tree label{margin-bottom:0;cursor:pointer}.react-checkbox-tree label:hover{background:rgba(51,51,204,.1)}.react-checkbox-tree label:active{background:rgba(51,51,204,.15)}.react-checkbox-tree:not(.rct-native-display) input{display:none}.react-checkbox-tree.rct-native-display input{margin:0 5px}.react-checkbox-tree .rct-icon{font-family:FontAwesome;font-style:normal}.rct-disabled>.rct-text>label{opacity:.75;cursor:not-allowed}.rct-disabled>.rct-text>label:hover{background:transparent}.rct-disabled>.rct-text>label:active{background:transparent}.rct-text{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.rct-options{-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto;margin-left:.5rem;text-align:right}.rct-option{opacity:.75;border:0;background:none;cursor:pointer;padding:0 4px;font-size:18px}.rct-option:hover{opacity:1}.rct-option+.rct-option{margin-left:2px}.rct-checkbox,.rct-collapse,.rct-node-icon{padding:0 5px}.rct-checkbox *,.rct-collapse *,.rct-node-icon *{display:inline-block;margin:0;width:14px}.rct-collapse{border:0;background:none;line-height:normal;color:inherit;font-size:12px}.rct-collapse.rct-collapse-btn{cursor:pointer}.rct-collapse>.rct-icon-expand-close{opacity:.5}.rct-collapse>.rct-icon-expand-close:hover{opacity:1}.rct-native-display .rct-checkbox{display:none}.rct-node-clickable{cursor:pointer}.rct-node-clickable:hover{background:rgba(51,51,204,.1)}.rct-node-clickable:focus{outline:0;background:rgba(51,51,204,.2)}.rct-node-icon{color:#33c}.rct-title{padding:0 5px}.rct-icon-expand-close:before{content:"\F054"}.rct-icon-expand-open:before{content:"\F078"}.rct-icon-uncheck:before{content:"\F096"}.rct-icon-check:before{content:"\F046"}.rct-icon-half-check:before{opacity:.5;content:"\F046"}.rct-icon-leaf:before{content:"\F016"}.rct-icon-parent-open:before{content:"\F115"}.rct-icon-parent-close:before{content:"\F114"}.rct-icon-expand-all:before{content:"\F0FE"}.rct-icon-collapse-all:before{content:"\F146"}.chart-tooltip{opacity:.75;font-size:12px}.active .color-bar.badge-20,.dashboard-component-chart-holder:hover .color-bar.badge-20,.dashboard-filter-indicators-container:hover .color-bar.badge-20,.filter-badge.badge-20{background-color:#087f5b}.active .color-bar.badge-19,.dashboard-component-chart-holder:hover .color-bar.badge-19,.dashboard-filter-indicators-container:hover .color-bar.badge-19,.filter-badge.badge-19{background-color:#5c940d}.active .color-bar.badge-18,.dashboard-component-chart-holder:hover .color-bar.badge-18,.dashboard-filter-indicators-container:hover .color-bar.badge-18,.filter-badge.badge-18{background-color:#c92a2a}.active .color-bar.badge-17,.dashboard-component-chart-holder:hover .color-bar.badge-17,.dashboard-filter-indicators-container:hover .color-bar.badge-17,.filter-badge.badge-17{background-color:#5f3dc4}.active .color-bar.badge-16,.dashboard-component-chart-holder:hover .color-bar.badge-16,.dashboard-filter-indicators-container:hover .color-bar.badge-16,.filter-badge.badge-16{background-color:#0b7285}.active .color-bar.badge-15,.dashboard-component-chart-holder:hover .color-bar.badge-15,.dashboard-filter-indicators-container:hover .color-bar.badge-15,.filter-badge.badge-15{background-color:#a61e4d}.active .color-bar.badge-14,.dashboard-component-chart-holder:hover .color-bar.badge-14,.dashboard-filter-indicators-container:hover .color-bar.badge-14,.filter-badge.badge-14{background-color:#d9480f}.active .color-bar.badge-13,.dashboard-component-chart-holder:hover .color-bar.badge-13,.dashboard-filter-indicators-container:hover .color-bar.badge-13,.filter-badge.badge-13{background-color:#e67700}.active .color-bar.badge-12,.dashboard-component-chart-holder:hover .color-bar.badge-12,.dashboard-filter-indicators-container:hover .color-bar.badge-12,.filter-badge.badge-12{background-color:#2b8a3e}.active .color-bar.badge-11,.dashboard-component-chart-holder:hover .color-bar.badge-11,.dashboard-filter-indicators-container:hover .color-bar.badge-11,.filter-badge.badge-11{background-color:#1864ab}.active .color-bar.badge-10,.dashboard-component-chart-holder:hover .color-bar.badge-10,.dashboard-filter-indicators-container:hover .color-bar.badge-10,.filter-badge.badge-10{background-color:#12b886}.active .color-bar.badge-9,.dashboard-component-chart-holder:hover .color-bar.badge-9,.dashboard-filter-indicators-container:hover .color-bar.badge-9,.filter-badge.badge-9{background-color:#74b816}.active .color-bar.badge-8,.dashboard-component-chart-holder:hover .color-bar.badge-8,.dashboard-filter-indicators-container:hover .color-bar.badge-8,.filter-badge.badge-8{background-color:#fa5252}.active .color-bar.badge-7,.dashboard-component-chart-holder:hover .color-bar.badge-7,.dashboard-filter-indicators-container:hover .color-bar.badge-7,.filter-badge.badge-7{background-color:#7950f2}.active .color-bar.badge-6,.dashboard-component-chart-holder:hover .color-bar.badge-6,.dashboard-filter-indicators-container:hover .color-bar.badge-6,.filter-badge.badge-6{background-color:#15aabf}.active .color-bar.badge-5,.dashboard-component-chart-holder:hover .color-bar.badge-5,.dashboard-filter-indicators-container:hover .color-bar.badge-5,.filter-badge.badge-5{background-color:#e64980}.active .color-bar.badge-4,.dashboard-component-chart-holder:hover .color-bar.badge-4,.dashboard-filter-indicators-container:hover .color-bar.badge-4,.filter-badge.badge-4{background-color:#f76707}.active .color-bar.badge-3,.dashboard-component-chart-holder:hover .color-bar.badge-3,.dashboard-filter-indicators-container:hover .color-bar.badge-3,.filter-badge.badge-3{background-color:#fab005}.active .color-bar.badge-2,.dashboard-component-chart-holder:hover .color-bar.badge-2,.dashboard-filter-indicators-container:hover .color-bar.badge-2,.filter-badge.badge-2{background-color:#40c057}.active .color-bar.badge-1,.dashboard-component-chart-holder:hover .color-bar.badge-1,.dashboard-filter-indicators-container:hover .color-bar.badge-1,.filter-badge.badge-1{background-color:#228be6}.dashboard{position:relative;color:#263238}.dashboard-header{background:#fff;display:flex;flex-direction:row;align-items:center;justify-content:space-between;padding:0 24px;box-shadow:0 4px 4px 0 rgba(0,0,0,.1)}.dashboard-content{display:flex;flex-direction:row;flex-wrap:nowrap;height:auto}.dashboard>.with-popover-menu>.popover-menu{left:24px}.dashboard .dashboard-component-tabs{box-shadow:0 4px 4px 0 rgba(0,0,0,.1);padding-left:8px}.dashboard-content .grid-container .dashboard-component-tabs{box-shadow:none;padding-left:0}.dashboard-content>div:first-child{width:100%;flex-grow:1;position:relative}.dropdown-toggle.btn.btn-primary .caret{color:#fff}.background--transparent{background-color:transparent}.background--white{background-color:#fff}.dashboard-builder-sidepane{flex:0 0 374px;position:relative}.dashboard-builder-sidepane .dashboard-builder-sidepane-header{font-size:16px;font-weight:700;border-top:1px solid #cfd8dc;border-bottom:1px solid #cfd8dc;padding:16px;display:flex;align-items:center}.dashboard-builder-sidepane .trigger{font-size:16px;color:#263238;opacity:1;margin-left:auto;cursor:pointer}.dashboard-builder-sidepane .slices-layer .trigger{margin-left:0;margin-right:20px}.dashboard-builder-sidepane .viewport{position:absolute;transform:none!important;overflow:hidden;width:374px;height:100%;box-shadow:-4px 0 4px 0 rgba(0,0,0,.1)}.dashboard-builder-sidepane .slider-container{position:absolute;background:#fff;width:748px;height:100vh;display:flex;transition:all .3s ease}.dashboard-builder-sidepane .slider-container.slide-in{left:-374px}.dashboard-builder-sidepane .slider-container.slide-out{left:0}.dashboard-builder-sidepane .slider-container .slide-content{width:374px}.dashboard-builder-sidepane .component-layer .new-component.static,.dashboard-builder-sidepane .slices-layer .dashboard-builder-sidepane-header{cursor:pointer}.dashboard-builder-sidepane .new-component-label{flex-grow:1}.dashboard-builder-sidepane .chart-card-container .chart-card{border:1px solid #cfd8dc;font-weight:200;padding:16px;margin:0 16px 16px;position:relative;cursor:move;background:hsla(0,0%,100%,.35)}.dashboard-builder-sidepane .chart-card-container .chart-card:hover{background:#f5f5f5}.dashboard-builder-sidepane .chart-card-container .card-title{margin-right:60px;margin-bottom:8px;font-weight:700}.dashboard-builder-sidepane .chart-card-container .card-body{display:flex;flex-direction:column}.dashboard-builder-sidepane .chart-card-container .card-body .item span{word-break:break-all}.dashboard-builder-sidepane .chart-card-container .card-body .item span:first-child{font-weight:400}.dashboard-builder-sidepane .chart-card-container .chart-card.is-selected{cursor:not-allowed;opacity:.4}.dashboard-builder-sidepane .chart-card-container .is-added-label{background:#263238;color:#fff;font-size:12px;line-height:1;text-transform:uppercase;position:absolute;padding:4px 8px;top:32px;right:32px;pointer-events:none}.dashboard-builder-sidepane .slice-adder-container{position:relative;min-height:200px}.dashboard-builder-sidepane .slice-adder-container .error-message{padding:16px}.dashboard-builder-sidepane .slice-adder-container .controls{display:flex;padding:16px}.dashboard-builder-sidepane .slice-adder-container .controls .search-input{flex-grow:1;margin-right:16px}.dashboard-builder-sidepane .slice-adder-container .controls .dropdown.btn-group button,.dashboard-builder-sidepane .slice-adder-container .controls input{font-size:14px;padding:7px 12px;height:32px;border:1px solid #cfd8dc}.dashboard-builder-sidepane .slice-adder-container .controls input{width:100%}.dashboard-builder-sidepane .slice-adder-container .controls input:focus{outline:none;border-color:#879399}.dashboard-builder-sidepane .slice-adder-container .ReactVirtualized__Grid.ReactVirtualized__List:focus{outline:none}.dashboard-builder-sidepane .color-scheme-container{list-style:none;margin:0;padding:0;display:flex;align-items:center}.dashboard-builder-sidepane .color-scheme-container li{flex-basis:9px;height:10px;margin:9px 1px}.icon-button{color:#879399;font-size:16px;display:flex;flex-direction:row;align-items:center;justify-content:center;outline:none}.icon-button:active,.icon-button:focus,.icon-button:hover{color:#263238;outline:none;text-decoration:none}.icon-button-label{color:#333;padding-left:8px;font-size:14px}.text-button{outline:none;border:none;margin:0;padding:0;background:none;text-decoration:none;font-size:inherit;font-weight:inherit}header.top{margin-bottom:-20px}body h1{font-size:28px;letter-spacing:-.2px;margin-bottom:12px}body h1,body h2{font-weight:700;line-height:1.4;margin-top:12px}body h2{font-size:21px;margin-bottom:8px}body h3,body h4,body h5,body h6{font-weight:700;line-height:1.4;font-size:16px;letter-spacing:.2px;margin-top:8px;margin-bottom:4px}body p{margin:0 0 8px}.dashboard .chart-header{position:relative;font-size:16px;font-weight:700;margin-bottom:4px}.dashboard .chart-header .dropdown.btn-group{position:absolute;right:0}.dashboard .chart-header .dropdown-toggle.btn.btn-default{background:none;border:none;box-shadow:none}.dashboard .chart-header .dropdown-menu.dropdown-menu-right{top:20px}.dashboard .chart-header .divider{margin:5px 0}.dashboard .chart-header .refresh-tooltip{display:block;height:16px;margin:3px 0;color:#879399}.dashboard .dashboard-header #save-dash-split-button{border-radius:0;margin-left:-9px;height:30px;width:30px}.dashboard .dashboard-header #save-dash-split-button.btn.btn-primary{border-left-color:#fff}.dashboard .dashboard-header #save-dash-split-button+.dropdown-menu.dropdown-menu-right{min-width:unset}.dashboard .dashboard-header #save-dash-split-button .caret{display:inline-block;width:100%;height:100%}.dashboard .dashboard-header .button-container{display:flex;flex-direction:row;flex-wrap:nowrap}.dashboard .dashboard-header .button-container>:nth-child(3){border-radius:4px 0 0 4px;border-right:none}.dashboard .dashboard-header .button-container>:nth-child(4){border-radius:0 4px 4px 0}.dashboard .dashboard-header .button-container>:not(:nth-child(3)):not(:last-child){margin-right:8px}.dashboard .chart-header .dropdown-menu,.dashboard .dashboard-header .dropdown-menu{padding:9px 0}.dashboard .chart-header .dropdown-menu li a,.dashboard .dashboard-header .dropdown-menu li a{padding:3px 16px;color:#263238;font-size:14px}.dashboard .chart-header .dropdown-menu li a:focus,.dashboard .chart-header .dropdown-menu li a:hover,.dashboard .dashboard-header .dropdown-menu li a:focus,.dashboard .dashboard-header .dropdown-menu li a:hover{background:#f2f3f5;color:#263238}.modal .modal-body{padding:24px 24px 29px}.modal .modal-dialog.filter-scope-modal{width:80%}.modal .refresh-warning-container{margin-top:24px}.modal .dashboard-modal-actions-container{margin-top:24px;text-align:right}.modal .dashboard-modal-actions-container .btn{margin-right:16px}.modal .dashboard-modal-actions-container .btn:last-child{margin-right:0}.modal .dashboard-modal.delete .btn.btn-primary{background:#e04355;border-color:#e04355}.react-bs-container-body{max-height:400px;overflow-y:auto}#pageDropDown,.hidden{display:none}.separator .chart-container{position:absolute;left:0;right:0;top:0;bottom:0}.dashboard .title{margin:0 20px}.dashboard-header .dashboard-component-header{display:flex;flex-direction:row;align-items:center}.dashboard-header .dashboard-component-header .favstar{font-size:21px;position:relative;margin-left:8px}.dashboard-header .dashboard-component-header .publish{position:relative;margin-left:8px}.slice_container .alert{margin:10px}i.danger{color:#e04355}i.warning{color:#ff7f44}.dragdroppable{position:relative}.dragdroppable--dragging{opacity:.2}.dragdroppable-row{width:100%}.drop-indicator{display:block;background-color:#20a7c9;position:absolute;z-index:10}.drop-indicator--top{top:0;left:0;height:4px;width:100%;min-width:16px}.drop-indicator--bottom{top:100%;left:0;height:4px;width:100%;min-width:16px}.drop-indicator--right{top:0;left:100%;height:100%;width:4px;min-height:16px}.drop-indicator--left{top:0;left:0;height:100%;width:4px;min-height:16px}.drag-handle{overflow:hidden;width:16px;cursor:move}.drag-handle--left{width:8px}.drag-handle-dot{float:left;height:2px;margin:1px;width:2px}.drag-handle-dot:after{content:"";background:#879399;float:left;height:2px;margin:-1px;width:2px}.dashboard-component-tabs-content>.empty-droptarget{position:absolute;width:100%}.dashboard-component-tabs-content>.empty-droptarget:first-child{height:16px;top:-8px;z-index:10}.dashboard-component-tabs-content>.empty-droptarget:last-child{height:12px;bottom:0}.grid-content>.empty-droptarget{width:100%;height:100%}.grid-content>.empty-droptarget:first-child{height:24px;margin-top:-24px}.filter-scope-container{display:flex;flex-direction:column;height:80%;margin-right:-24px;font-size:14px}.filter-scope-container .nav.nav-tabs{border:none}.filter-scope-container .filter-scope-body{flex:1;max-height:calc(100% - 128px)}.filter-scope-container .filter-scope-body .filter-field-pane,.filter-scope-container .filter-scope-body .filter-scope-pane{overflow-y:scroll}.filter-scope-container .dashboard-modal-actions-container{height:64px;border-top:1px solid #cfd8dc;padding:24px;margin:0 0 0 -24px}.filter-scope-container .warning-message{padding:24px}.filter-scope-header{height:64px;border-bottom:1px solid #cfd8dc;padding-left:24px;margin-left:-24px}.filter-scope-header h4{margin-top:0}.filter-scope-header .selected-fields{margin:12px 0 16px;visibility:hidden}.filter-scope-header .selected-fields.multi-edit-mode{visibility:visible}.filter-scope-header .selected-fields .selected-scopes{padding-left:5px}.filters-scope-selector{display:flex;flex-direction:row;position:relative;height:100%}.filters-scope-selector a,.filters-scope-selector a:active,.filters-scope-selector a:hover{color:#263238;text-decoration:none}.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-collapse-all,.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-expand-all{font-size:14px;font-family:Inter,Helvetica,Arial;color:#20a7c9}.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-collapse-all:before,.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-expand-all:before{content:""}.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-collapse-all:hover,.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-expand-all:hover{text-decoration:underline}.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-collapse-all:focus,.filters-scope-selector .react-checkbox-tree .rct-icon.rct-icon-expand-all:focus{outline:none}.filters-scope-selector .filter-field-pane{position:relative;width:40%;padding:16px 16px 16px 0;border-right:1px solid #cfd8dc}.filters-scope-selector .filter-field-pane .filter-container label{font-weight:400;margin:0 0 0 16px;word-break:break-all}.filters-scope-selector .filter-field-pane .filter-field-item{height:35px;display:flex;align-items:center;padding:0 24px;margin-left:-24px}.filters-scope-selector .filter-field-pane .filter-field-item.is-selected{border:1px solid #a3a3a3;border-radius:4px;background-color:#f5f5f5;margin-left:-25px}.filters-scope-selector .filter-field-pane .react-checkbox-tree .rct-title .root{font-weight:700}.filters-scope-selector .filter-field-pane .react-checkbox-tree .rct-text{height:40px}.filters-scope-selector .filter-scope-pane{position:relative;flex:1;padding:16px 24px 16px 16px}.filters-scope-selector .react-checkbox-tree{flex-direction:column;color:#263238;font-size:14px}.filters-scope-selector .react-checkbox-tree .filter-scope-type{padding:8px 0;display:block}.filters-scope-selector .react-checkbox-tree .filter-scope-type .type-indicator{position:relative;top:3px;margin-right:8px}.filters-scope-selector .react-checkbox-tree .filter-scope-type.chart{font-weight:400}.filters-scope-selector .react-checkbox-tree .filter-scope-type.selected-filter{padding-left:28px;position:relative;color:#a3a3a3}.filters-scope-selector .react-checkbox-tree .filter-scope-type.selected-filter:before{content:" ";position:absolute;left:0;top:50%;width:18px;height:18px;border-radius:4px;margin-top:-9px;box-shadow:inset 0 0 0 2px #cfd8dc;background:#f2f2f2}.filters-scope-selector .react-checkbox-tree .filter-scope-type.root{font-weight:700}.filters-scope-selector .react-checkbox-tree .rct-checkbox svg{position:relative;top:3px;width:18px}.filters-scope-selector .react-checkbox-tree .rct-node-leaf .rct-bare-label:before{padding-left:5px}.filters-scope-selector .react-checkbox-tree .rct-options{text-align:left;margin-left:0;margin-bottom:8px}.filters-scope-selector .react-checkbox-tree .rct-text{margin:0;display:flex}.filters-scope-selector .react-checkbox-tree .rct-title{display:block}.filters-scope-selector .react-checkbox-tree .rct-node-clickable:focus,.filters-scope-selector .react-checkbox-tree .rct-node-clickable:hover,.filters-scope-selector .react-checkbox-tree label:active,.filters-scope-selector .react-checkbox-tree label:hover{background:none!important}.filters-scope-selector .multi-edit-mode.filter-scope-pane .rct-node.rct-node-leaf .filter-scope-type.filter_box{display:none}.filters-scope-selector .multi-edit-mode .filter-field-item{padding:0 16px 0 50px;margin-left:-50px}.filters-scope-selector .multi-edit-mode .filter-field-item.is-selected{margin-left:-51px}.filters-scope-selector .scope-search{position:absolute;right:16px;top:16px;border-radius:4px;border:1px solid #cfd8dc;padding:4px 8px;font-size:14px;outline:none}.filters-scope-selector .scope-search:focus{border:1px solid #20a7c9}.dashboard-filter-indicators-container{position:absolute;right:-20px;top:40px;width:20px;height:125px}.dashboard-filter-indicators-container .indicator-container{position:relative;margin-bottom:4px}.dashboard-filter-indicators-container .filter-indicator,.dashboard-filter-indicators-container .filter-indicator-group{width:3px;height:20px;overflow:hidden;display:flex;background-color:#cfd8dc;transition:width .3s;border-top-right-radius:2px;border-bottom-right-radius:2px}.dashboard-filter-indicators-container .filter-indicator-group .color-bar,.dashboard-filter-indicators-container .filter-indicator .color-bar{width:0;height:20px;position:absolute;right:100%;transition:width .3s}.dashboard-filter-indicators-container .filter-indicator-group .filter-badge,.dashboard-filter-indicators-container .filter-indicator .filter-badge{width:20px;height:20px}.dashboard-filter-indicators-container .filter-indicator-group{box-shadow:-2px 0 0 0 #fff,-4px 0 0 0 #cfd8dc}.show-outline .filter-indicator-group{box-shadow:-2px 0 0 0 #20a7c9,-4px 0 0 0 #fff,-6px 0 0 0 #cfd8dc}.dashboard-component-chart-holder .active.filter-indicator,.dashboard-component-chart-holder .active.filter-indicator-group,.dashboard-component-chart-holder:hover .filter-indicator,.dashboard-component-chart-holder:hover .filter-indicator-group,.dashboard-filter-indicators-container .active.filter-indicator,.dashboard-filter-indicators-container .active.filter-indicator-group,.dashboard-filter-indicators-container:hover .filter-indicator,.dashboard-filter-indicators-container:hover .filter-indicator-group{width:20px;background-color:transparent}.dashboard-component-chart-holder .active.filter-indicator-group .color-bar,.dashboard-component-chart-holder .active.filter-indicator .color-bar,.dashboard-component-chart-holder:hover .filter-indicator-group .color-bar,.dashboard-component-chart-holder:hover .filter-indicator .color-bar,.dashboard-filter-indicators-container .active.filter-indicator-group .color-bar,.dashboard-filter-indicators-container .active.filter-indicator .color-bar,.dashboard-filter-indicators-container:hover .filter-indicator-group .color-bar,.dashboard-filter-indicators-container:hover .filter-indicator .color-bar{width:2px}.dashboard-component-chart-holder .active.filter-indicator-group .filter-badge,.dashboard-component-chart-holder .active.filter-indicator .filter-badge,.dashboard-component-chart-holder:hover .filter-indicator-group .filter-badge,.dashboard-component-chart-holder:hover .filter-indicator .filter-badge,.dashboard-filter-indicators-container .active.filter-indicator-group .filter-badge,.dashboard-filter-indicators-container .active.filter-indicator .filter-badge,.dashboard-filter-indicators-container:hover .filter-indicator-group .filter-badge,.dashboard-filter-indicators-container:hover .filter-indicator .filter-badge{border-top-left-radius:0;border-bottom-left-radius:0}.dashboard-component-chart-holder .filter-indicator-group,.dashboard-filter-indicators-container .filter-indicator-group{box-shadow:-4px 0 0 0 #fff,-6px 0 0 0 #cfd8dc}#filter-indicator-tooltip{font-size:14px;text-align:left}#filter-indicator-tooltip>.tooltip-arrow{margin-right:8px;border-left-color:#333}#filter-indicator-tooltip>.tooltip-inner{width:200px;max-width:200px;border-radius:4px;margin-right:8px;padding:16px 12px;background-color:#333;text-align:left}.tooltip-item{position:relative}.tooltip-item .filter-content{margin-right:22px;text-align:left;display:flex;flex-flow:row wrap;align-items:stretch}.tooltip-item .filter-content label{font-weight:700;white-space:nowrap;text-overflow:ellipsis;overflow:hidden;margin-right:4px}.tooltip-item .filter-edit{cursor:pointer;position:absolute;top:4px;right:0}.group-title{margin-bottom:8px}.tooltip-group{margin:0;padding:0;list-style:none}.tooltip-group li{margin-bottom:8px}.grid-container{position:relative;margin:24px 36px;min-width:0;width:100%}.grid-container>div:first-child{height:inherit!important}.grid-content{display:flex;flex-direction:column}.grid-content>div:not(:only-child):not(:last-child):not(.empty-droptarget){margin-bottom:16px}.grid-column-guide{top:0;min-height:100%;background-color:rgba(32,167,201,.1);box-shadow:inset 0 0 0 1px rgba(32,167,201,.6)}.grid-column-guide,.grid-row-guide{position:absolute;pointer-events:none}.grid-row-guide{left:0;bottom:2;height:2;background-color:#20a7c9;z-index:10}.hover-menu{opacity:0;position:absolute;z-index:10;font-size:14px}.hover-menu--left{width:24px;top:50%;transform:translateY(-50%);left:-24px;padding:8px 0;display:flex;flex-direction:column;justify-content:center;align-items:center}.hover-menu--left>:nth-child(n):not(:only-child):not(:last-child){margin-bottom:12px}.hover-menu--top{height:24px;top:-24px;left:50%;transform:translate(-50%);padding:0 8px;display:flex;flex-direction:row;justify-content:center;align-items:center}.dragdroppable-column .dragdroppable-row .hover-menu--left{left:-12px;background:#fff;border:1px solid #cfd8dc}.dashboard-component-tabs .dragdroppable-column .hover-menu--top,.dragdroppable-column .dragdroppable-column .hover-menu--top{top:-12px;background:#fff;border:1px solid #cfd8dc}.dashboard-component-tabs>.hover-menu--left{top:0;transform:unset;background:transparent}.dragdroppable-column .dashboard-component-chart-holder>.hover-menu--top{right:8px;top:8px;background:transparent;border:none;transform:unset;left:unset}.hover-menu--top>:nth-child(n):not(:only-child):not(:last-child){margin-right:12px}.hover-menu:hover,div:hover>.hover-menu{opacity:1}.with-popover-menu{position:relative;outline:none}.grid-row.grid-row--empty .with-popover-menu{width:100%;height:100%}.with-popover-menu--focused:after{content:"";position:absolute;top:1;left:-1;width:100%;height:100%;box-shadow:inset 0 0 0 2px #20a7c9;pointer-events:none}.popover-menu{position:absolute;flex-wrap:nowrap;left:1px;top:-42px;height:40px;padding:0 16px;background:#fff;box-shadow:0 1px 2px 1px rgba(0,0,0,.35);font-size:14px;cursor:default;z-index:3000}.popover-menu,.popover-menu .menu-item{display:flex;flex-direction:row;align-items:center}.popover-menu .menu-item:not(:only-child):not(:last-child):after{content:"";width:1;height:100%;background:#cfd8dc;margin:0 16px}.popover-menu .popover-dropdown.btn{border:none;padding:0;font-size:inherit;color:#263238}.dashboard-component-tabs li .with-popover-menu--focused:after{top:-12px;left:-8px;width:calc(100% + 16px);height:calc(100% + 28px)}.dashboard-component-tabs li .popover-menu{top:-56px;left:-7px}.hover-dropdown .btn:active,.hover-dropdown .btn:focus,.hover-dropdown .btn:hover,.popover-menu .popover-dropdown.btn:active,.popover-menu .popover-dropdown.btn:focus,.popover-menu .popover-dropdown.btn:hover{background:initial;box-shadow:none}.hover-dropdown li.dropdown-item:hover a,.popover-menu li.dropdown-item:hover a{background:#f2f3f5}.hover-dropdown li.dropdown-item.active a,.popover-menu li.dropdown-item.active a{background:#cfd8dc;font-weight:700;color:#263238}.popover-dropdown .caret{width:auto;border-top-color:transparent}.background-style-option{display:inline-block}.background-style-option:before{content:"";width:1em;height:1em;margin-right:8px;display:inline-block;vertical-align:middle}.background-style-option.background--white{padding-left:0;background:transparent}.background-style-option.background--white:before{background:#fff;border:1px solid #cfd8dc}.background-style-option.background--transparent:before{background-image:linear-gradient(45deg,#879399 25%,transparent 0),linear-gradient(-45deg,#879399 25%,transparent 0),linear-gradient(45deg,transparent 75%,#879399 0),linear-gradient(-45deg,transparent 75%,#879399 0);background-size:8px 8px;background-position:0 0,0 4px,4px -4px,-4px 0}.resizable-container{background-color:transparent;position:relative}.resizable-container~div{width:auto!important;height:auto!important}.resizable-container--resizing:after{content:"";position:absolute;top:0;left:0;width:100%;height:100%;box-shadow:inset 0 0 0 2px #20a7c9}.resizable-container--resizing>span .resize-handle{border-color:#20a7c9}.resize-handle{opacity:0;z-index:10}.resize-handle--bottom-right{position:absolute;border:solid;border-width:0 1.5px 1.5px 0;border-right-color:#879399;border-bottom-color:#879399;right:16px;bottom:16px;width:8px;height:8px}.resize-handle--right{width:2px;height:20px;right:4px;top:50%;transform:translateY(-50%);position:absolute;border-left:1px solid #879399;border-right:1px solid #879399}.resize-handle--bottom{height:2px;width:20px;bottom:4px;left:50%;transform:translate(-50%);position:absolute;border-top:1px solid #879399;border-bottom:1px solid #879399}.resizable-container--resizing .resize-handle,.resizable-container:hover .resize-handle{opacity:1}.dragdroppable-column .resizable-container-handle--right{right:-10px!important}.dragdroppable-column .dragdroppable-column .resizable-container-handle--right{right:0!important}.resizable-container-handle--bottom{bottom:0!important}.dashboard-component-chart-holder{width:100%;height:100%;color:#333;background-color:#fff;position:relative;padding:16px}.dashboard-component-chart-holder .missing-chart-container{display:flex;flex-direction:column;align-items:center;overflow-y:auto;justify-content:center}.dashboard-component-chart-holder .missing-chart-container .missing-chart-body{font-size:12px;position:relative;display:flex}.dashboard-component-chart-holder.fade-in{border-radius:4px;box-shadow:inset 0 0 0 2px #20a7c9,0 0 0 3px rgba(32,167,201,.1);transition:box-shadow 1s ease-in-out}.dashboard-component-chart-holder.fade-out{border-radius:4px;box-shadow:none;transition:box-shadow 1s ease-in-out}.dashboard-chart{overflow:hidden}.dashboard-chart.dashboard-chart--overflowable{overflow:visible}.dashboard--editing .dashboard-component-chart-holder:after{content:"";position:absolute;width:100%;height:100%;top:0;left:0;z-index:1;pointer-events:none;border:1px solid transparent}.dashboard--editing .dashboard-component-chart-holder:hover:after{border:1px dashed #20a7c9;z-index:2}.dashboard--editing .resizable-container.resizable-container--resizing:hover>.dashboard-component-chart-holder:after,.dashboard--editing .resizable-container:hover>.dashboard-component-chart-holder:after{border:1px dashed #20a7c9}.dashboard--editing .resizable-container .dashboard-component-chart-holder .dashboard-chart .chart-container{cursor:move;opacity:.2}.dashboard--editing .resizable-container .dashboard-component-chart-holder .dashboard-chart .slice_container{pointer-events:none}.dashboard--editing .resizable-container .dashboard-component-chart-holder:hover .dashboard-chart .chart-container{opacity:.7}.slice-header-controls-trigger{padding:0 16px;position:absolute;top:0;right:-16px}.slice-header-controls-trigger:hover{cursor:pointer}.dot{height:4px;width:4px;border-radius:2px;margin:2px 0;background-color:#879399;display:inline-block}.vertical-dots-container .dot{display:block}a[role=menuitem] .dot{width:8px;height:8px;margin-right:8px}.time-filter-tabs>.nav-tabs{margin-bottom:8px}.time-filter-tabs>.nav-tabs>li>a{padding:4px}.full-size{position:fixed;z-index:3000;left:0;top:0}.grid-column{width:100%;position:relative}.grid-column>:not(:only-child):not(.hover-menu):not(:last-child){margin-bottom:16px}.dashboard--editing .grid-column:after{content:"";position:absolute;width:100%;height:100%;top:0;left:0;z-index:1;pointer-events:none;border:1px dashed #cfd8dc}.dashboard--editing .hover-menu:hover+.grid-column:after,.dashboard--editing .resizable-container.resizable-container--resizing:hover>.grid-column:after{border:1px dashed #20a7c9;z-index:2}.grid-column--empty{min-height:100px}.grid-column--empty:before{content:"Empty column";position:absolute;top:0;left:0;width:100%;height:100%;display:flex;align-items:center;justify-content:center;color:#cfd8dc}.dashboard-component-divider{width:100%;padding:8px 0;background-color:transparent}.dashboard-component-divider:after{content:"";height:1px;width:100%;background-color:#cfd8dc;display:block}.new-component-placeholder.divider-placeholder:after{content:"";height:2px;width:100%;background-color:#cfd8dc}.dragdroppable .dashboard-component-divider{cursor:move}.dashboard-component-header{width:100%;font-weight:700;padding:16px 0;color:#263238}.dashboard--editing .dashboard-grid .dashboard-component-header:after{border:1px dashed transparent;content:"";position:absolute;width:100%;height:100%;top:0;left:0;z-index:1;pointer-events:none}.dashboard--editing .dashboard-grid .dashboard-component-header:hover:after{border:1px dashed #20a7c9;z-index:2}.dashboard--editing .dragdroppable-row .dashboard-component-header{cursor:move}.dashboard--editing .header-style-option{font-weight:700;color:#263238}.dashboard-header .dashboard-component-header{font-weight:200;width:auto}.dashboard--editing .header-small{font-size:16px}.header-medium{font-size:21px}.header-large{font-size:28px}.background--white .dashboard-component-header,.dashboard-component-header.background--white,.dashboard-component-tabs .dashboard-component-divider,.dashboard-component-tabs .dashboard-component-header{padding-left:16px;padding-right:16px}.grid-column>:not(:only-child):not(:last-child) .dashboard-component-header{margin-bottom:-16px}.new-component{display:flex;flex-direction:row;flex-wrap:nowrap;align-items:center;padding:16px;background:#fff;cursor:move}.new-component:not(.static):hover{background:#f5f5f5}.new-component-placeholder{position:relative;background:#f5f5f5;width:40px;height:40px;margin-right:16px;border:1px solid #fff;display:flex;align-items:center;justify-content:center;color:#879399;font-size:28px}.new-component-placeholder.fa-window-restore{font-size:16px}.new-component-placeholder.fa-area-chart{font-size:21px}.grid-row{position:relative;display:flex;flex-direction:row;flex-wrap:nowrap;align-items:flex-start;width:100%;height:fit-content}.grid-row>:not(:only-child):not(:last-child):not(.hover-menu){margin-right:16px}.dashboard--editing .dashboard-component-tabs>.hover-menu:hover+div:after,.dashboard--editing .grid-row:after{border:1px dashed transparent;content:"";position:absolute;width:100%;height:100%;top:0;left:0;z-index:1;pointer-events:none}.dashboard--editing .dashboard-component-tabs>.hover-menu:hover+div:after,.dashboard--editing .hover-menu:hover+.grid-row:after,.dashboard--editing .resizable-container.resizable-container--resizing:hover>.grid-row:after{border:1px dashed #20a7c9;z-index:2}.dashboard--editing .dashboard-component-tabs>.hover-menu+div:after,.dashboard--editing .grid-row:after{border:1px dashed #cfd8dc}.dashboard--editing .dashboard-component-tabs-content .dragdroppable-row{padding-top:16px}.grid-row.grid-row--empty{align-items:center;height:100px}.grid-row.grid-row--empty:before{position:absolute;top:0;left:0;content:"Empty row";display:flex;align-items:center;justify-content:center;width:100%;height:100%;color:#879399}.dashboard-component-tabs{width:100%;background-color:#fff}.dashboard-component-tabs .nav-tabs{border-bottom:none}.dashboard-component-tabs .nav-tabs>li{margin:0 16px}.dashboard-component-tabs .nav-tabs>li>a{color:#263238;border:none;padding:12px 0 14px;font-size:14px;margin-right:0}.dashboard-component-tabs .nav-tabs>li>a:hover{border:none;background:inherit;color:#263238}.dashboard-component-tabs .nav-tabs>li>a:focus{outline:none;background:#fff}.dashboard-component-tabs .nav-tabs>li .dragdroppable-tab{cursor:move}.dashboard-component-tabs .nav-tabs>li .drop-indicator{top:-12px!important;height:calc(100% + 24px)!important}.dashboard-component-tabs .nav-tabs>li .drop-indicator--left{left:-12px!important}.dashboard-component-tabs .nav-tabs>li .drop-indicator--right{right:-12px!important}.dashboard-component-tabs .nav-tabs>li .drop-indicator--bottom,.dashboard-component-tabs .nav-tabs>li .drop-indicator--top{left:-12px!important;width:calc(100% + 24px)!important;opacity:.4}.dashboard-component-tabs .nav-tabs>li .fa-plus{color:#333;font-size:14px;margin-top:3px}.dashboard-component-tabs .nav-tabs>li .editable-title input[type=button]{cursor:pointer}.dashboard-component-tabs .nav-tabs li.active>a{border:none}.dashboard-component-tabs .nav-tabs li.active>a:after{content:"";position:absolute;height:3px;width:100%;bottom:0;background:linear-gradient(90deg,#20a7c9,#136479)}.dashboard-component-tabs .dashboard-component-tabs-content{min-height:48px;margin-top:1px;position:relative}.dashboard-markdown{overflow:hidden}.dashboard-markdown h4,.dashboard-markdown h5{font-weight:200}.dashboard-markdown h5{color:#a3a3a3}.dashboard-markdown h6{font-weight:400;font-size:12px}.dashboard-markdown .dashboard-component-chart-holder{overflow-y:auto;overflow-x:hidden}.dashboard--editing .dashboard-markdown{cursor:move}.dashboard-markdown #brace-editor{border:none}.with-popover-menu--focused .dashboard-markdown--editing .dashboard-component-chart-holder{padding:1px}