/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
@import '../stylesheets/less/variables.less';

.reactable-pagination td {
  padding: 15px 0 0 0 !important;
}

.reactable-pagination a:focus {
  text-decoration: none;
  color: @gray-dark;
  outline: 1;
}

.reactable-page-button,
.reactable-next-page,
.reactable-previous-page {
  background: @lightest;
  border-radius: @border-radius-normal;
  border: 1px solid @gray-light;
  color: @gray-dark;
  display: inline-block;
  font-size: @font-size-s;
  margin-right: 5px;
  padding: 5px 10px;
  text-align: center;
  text-decoration: none;
  vertical-align: middle;
  white-space: nowrap;

  &:hover {
    background-color: @gray-bg;
    border-color: @gray;
    color: @gray-dark;
    text-decoration: none;
  }
}

.reactable-current-page {
  border: 1px solid @gray-light;
  color: @gray-dark;
  font-weight: @font-weight-bold;
  pointer-events: none;
  opacity: 0.65;
}
