/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

@font-face {
  font-family: 'Fira Code';
  src: url('../../../fonts/FiraCode/woff2/FiraCode-Light.woff2') format('woff2'),
    url('../../../fonts/FiraCode/woff/FiraCode-Light.woff') format('woff');
  font-weight: 300;
  font-style: normal;
}

@font-face {
  font-family: 'Fira Code';
  src: url('../../../fonts/FiraCode/woff2/FiraCode-Regular.woff2')
      format('woff2'),
    url('../../../fonts/FiraCode/woff/FiraCode-Regular.woff') format('woff');
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: 'Fira Code';
  src: url('../../../fonts/FiraCode/woff2/FiraCode-Medium.woff2')
      format('woff2'),
    url('../../../fonts/FiraCode/woff/FiraCode-Medium.woff') format('woff');
  font-weight: 500;
  font-style: normal;
}

@font-face {
  font-family: 'Fira Code';
  src: url('../../../fonts/FiraCode/woff2/FiraCode-Bold.woff2') format('woff2'),
    url('../../../fonts/FiraCode/woff/FiraCode-Bold.woff') format('woff');
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: 'Fira Code VF';
  src: url('../../../fonts/FiraCode/woff2/FiraCode-VF.woff2')
      format('woff2-variations'),
    url('../../../fonts/FiraCode/woff/FiraCode-VF.woff')
      format('woff-variations');
  /* font-weight requires a range: https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Fonts/Variable_Fonts_Guide#Using_a_variable_font_font-face_changes */
  font-weight: 300 700;
  font-style: normal;
}
