.BootstrapSliderWrapper .slider-selection{background:#efefef}.BootstrapSliderWrapper .slider-handle{background:#b3b3b3}.play-slider{display:flex;height:40px;width:100%;margin:0}.play-slider-controls{flex:0 0 80px;text-align:middle}.play-slider-scrobbler{flex:1}.slider.slider-horizontal{width:100%!important}.slider-button{color:#b3b3b3;margin-right:5px}div.slider>div.tooltip.tooltip-main.top.in{margin-left:0!important}div.legend{font-size:90%;position:absolute;background:#fff;box-shadow:0 0 4px rgba(0,0,0,.15);margin:24px;padding:12px 20px;outline:none;overflow-y:scroll;max-height:200px}ul.categories{list-style:none;padding-left:0;margin:0}ul.categories li a{color:#333;text-decoration:none}ul.categories li a span{margin-right:10px}