{#
  Licensed to the Apache Software Foundation (ASF) under one
  or more contributor license agreements.  See the NOTICE file
  distributed with this work for additional information
  regarding copyright ownership.  The ASF licenses this file
  to you under the Apache License, Version 2.0 (the
  "License"); you may not use this file except in compliance
  with the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing,
  software distributed under the License is distributed on an
  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, either express or implied.  See the License for the
  specific language governing permissions and limitations
  under the License.
#}

{% set bug_report_url = appbuilder.app.config['BUG_REPORT_URL'] %}
{% set documentation_url = appbuilder.app.config['DOCUMENTATION_URL'] %}
{% set documentation_text = appbuilder.app.config['DOCUMENTATION_TEXT'] %}
{% set documentation_icon = appbuilder.app.config['DOCUMENTATION_ICON'] %}
{% set version_string = appbuilder.app.config['VERSION_STRING'] %}
{% set version_sha = appbuilder.app.config['VERSION_SHA'] %}

{% set locale = session['locale'] %}
{% if not locale %}
    {% set locale = 'en' %}
{% endif %}

{% if not current_user.is_anonymous %}
    <li class="dropdown">
        <button type="button" style="margin-top: 12px; margin-right: 30px;" data-toggle="dropdown" class="dropdown-toggle btn btn-sm btn-primary">
          <i class="fa fa-plus"></i> {{ _("New") }}
        </button>
        <ul class="dropdown-menu">
            <li><a href="/superset/sqllab"><span class="fa fa-fw fa-search"></span> {{_("SQL Query")}}</a></li>
            <li><a href="/chart/add"><span class="fa fa-fw fa-bar-chart"></span> {{_("Chart")}}</a></li>
            <li><a href="/dashboard/new/"><span class="fa fa-fw fa-dashboard"></span> {{_("Dashboard")}}</a></li>
        </ul>
    </li>
{% endif %}
{% if documentation_url %}
<li>
  <a
    tabindex="-1"
    href="{{ documentation_url }}"
    title="{{ documentation_text }}"
    target="_blank"
  >
  {% if documentation_icon %}
    <img
      width="100%"
      src="{{ documentation_icon }}"
      alt="{{ documentation_text }}"
    />
  {% else %}
    <i class="fa fa-question"></i>&nbsp;
  {% endif %}
  </a>
</li>
{% endif %}
{% if bug_report_url %}
<li>
  <a
    tabindex="-1"
    href="{{ bug_report_url }}"
    target="_blank"
    title="Report a bug"
  >
    <i class="fa fa-bug"></i>&nbsp;
  </a>
</li>
{% endif %}
{% if languages.keys()|length > 1 %}
<li class="dropdown">
    <a class="dropdown-toggle" data-toggle="dropdown" href="javascript:void(0)">
       <div class="f16"><i class="flag {{languages[locale].get('flag')}}"></i>&nbsp;<b class="caret"></b>
       </div>
    </a>
    <ul class="dropdown-menu" id="language-picker">
      <li class="dropdown">
        {% for lang in languages %}
            {% if lang != locale %}
                <a tabindex="-1" href="{{appbuilder.get_url_for_locale(lang)}}">
                  <div class="f16">
                    <i class="flag {{languages[lang].get('flag')}}"></i> - {{languages[lang].get('name')}}
                  </div>
                </a>
            {% endif %}
        {% endfor %}
      </li>
    </ul>
</li>
{% endif %}

{% if not current_user.is_anonymous %}
    <li class="dropdown">
      <a
        class="dropdown-toggle"
        data-toggle="dropdown"
        title="{{g.user.get_full_name()}}"
        href="javascript:void(0)"
      >
        <i class="fa fa-user"></i>&nbsp;<b class="caret"></b>
      </a>
        <ul class="dropdown-menu">
            <li><a href="{{appbuilder.get_url_for_userinfo}}"><span class="fa fa-fw fa-user"></span>{{_("Profile")}}</a></li>
            <li><a href="{{appbuilder.get_url_for_logout}}"><span class="fa fa-fw fa-sign-out"></span>{{_("Logout")}}</a></li>
            {% if version_string or version_sha %}
              <li class="fineprint">
                {% if version_string %}
                  <div>Version: {{version_string}}</div>
                {% endif %}
                {% if version_sha %}
                  <div>SHA: {{version_sha}}</div>
                {% endif %}
              </li>
            {% endif %}
        </ul>
    </li>
{% else %}
    <li><a href="{{appbuilder.get_url_for_login}}">
    <i class="fa fa-fw fa-sign-in"></i>{{_("Login")}}</a></li>
{% endif %}
